package com.huawei.dli.restapi.api;

import com.huawei.dli.restapi.ApiCallback;
import com.huawei.dli.restapi.ApiClient;
import com.huawei.dli.restapi.ApiException;
import com.huawei.dli.restapi.ApiResponse;
import com.huawei.dli.restapi.Configuration;
import com.huawei.dli.restapi.Pair;
import com.huawei.dli.restapi.ProgressRequestBody;
import com.huawei.dli.restapi.ProgressResponseBody;

import com.google.gson.reflect.TypeToken;

import java.io.IOException;


import com.huawei.dli.restapi.model.TokenAuthInfo;
import com.huawei.dli.restapi.model.TokenAuthResponse;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class TokenAuthApi {
    private ApiClient apiClient;

    public TokenAuthApi() {
        this(Configuration.getDefaultApiClient());
    }

    public TokenAuthApi(ApiClient apiClient) {
        this.apiClient = apiClient;
    }

    public ApiClient getApiClient() {
        return apiClient;
    }

    public void setApiClient(ApiClient apiClient) {
        this.apiClient = apiClient;
    }

    /**
     * Build call for getToken
     * @param tokenAuthInfo  (required)
     * @param progressListener Progress listener
     * @param progressRequestListener Progress request listener
     * @return Call to execute
     * @throws ApiException If fail to serialize the request body object
     */
    public okhttp3.Call getTokenCall(TokenAuthInfo tokenAuthInfo, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        Object localVarPostBody = tokenAuthInfo;
        
        // create path and map variables
        String localVarPath = "/v3/auth/tokens";

        if (ApiClient.isTest()) {
            localVarPath = localVarPath.replace("metastore/", "");
        }
        List<Pair> localVarQueryParams = new ArrayList<Pair>();
        List<Pair> localVarCollectionQueryParams = new ArrayList<Pair>();

        Map<String, String> localVarHeaderParams = new HashMap<String, String>();

        Map<String, Object> localVarFormParams = new HashMap<String, Object>();

        final String[] localVarAccepts = {
            
        };
        final String localVarAccept = apiClient.selectHeaderAccept(localVarAccepts);
        if (localVarAccept != null) localVarHeaderParams.put("Accept", localVarAccept);

        final String[] localVarContentTypes = {
            
        };
        final String localVarContentType = apiClient.selectHeaderContentType(localVarContentTypes);
        localVarHeaderParams.put("Content-Type", localVarContentType);

        if(progressListener != null) {
            apiClient.getHttpClient().networkInterceptors().add(new okhttp3.Interceptor() {
                @Override
                public okhttp3.Response intercept(okhttp3.Interceptor.Chain chain) throws IOException {
                    okhttp3.Response originalResponse = chain.proceed(chain.request());
                    return originalResponse.newBuilder()
                    .body(new ProgressResponseBody(originalResponse.body(), progressListener))
                    .build();
                }
            });
        }

        String[] localVarAuthNames = new String[] {  };
        return apiClient.buildCall(localVarPath, "POST", localVarQueryParams, localVarCollectionQueryParams, localVarPostBody, localVarHeaderParams, localVarFormParams, localVarAuthNames, progressRequestListener);
    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    private okhttp3.Call getTokenValidateBeforeCall(TokenAuthInfo tokenAuthInfo, final ProgressResponseBody.ProgressListener progressListener, final ProgressRequestBody.ProgressRequestListener progressRequestListener) throws ApiException {
        
        // verify the required parameter 'tokenAuthInfo' is set
        if (tokenAuthInfo == null) {
            throw new ApiException("Missing the required parameter 'tokenAuthInfo' when calling getToken(Async)");
        }
        
        
        okhttp3.Call call = getTokenCall(tokenAuthInfo, progressListener, progressRequestListener);
        return call;

    }

    /**
     * get token
     * get token for authentication
     * @param tokenAuthInfo  (required)
     * @return TokenAuthResponse
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public TokenAuthResponse getToken(TokenAuthInfo tokenAuthInfo) throws ApiException {
        ApiResponse<TokenAuthResponse> resp = getTokenWithHttpInfo(tokenAuthInfo);
        return resp.getData();
    }

    /**
     * get token
     * get token for authentication
     * @param tokenAuthInfo  (required)
     * @return ApiResponse&lt;TokenAuthResponse&gt;
     * @throws ApiException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ApiResponse<TokenAuthResponse> getTokenWithHttpInfo(TokenAuthInfo tokenAuthInfo) throws ApiException {
        okhttp3.Call call = getTokenValidateBeforeCall(tokenAuthInfo, null, null);
        Type localVarReturnType = new TypeToken<TokenAuthResponse>(){}.getType();
        return apiClient.execute(call, localVarReturnType);
    }

    /**
     * get token (asynchronously)
     * get token for authentication
     * @param tokenAuthInfo  (required)
     * @param callback The callback to be executed when the API call finishes
     * @return The request call
     * @throws ApiException If fail to process the API call, e.g. serializing the request body object
     */
    public okhttp3.Call getTokenAsync(TokenAuthInfo tokenAuthInfo, final ApiCallback<TokenAuthResponse> callback) throws ApiException {

        ProgressResponseBody.ProgressListener progressListener = null;
        ProgressRequestBody.ProgressRequestListener progressRequestListener = null;

        if (callback != null) {
            progressListener = new ProgressResponseBody.ProgressListener() {
                @Override
                public void update(long bytesRead, long contentLength, boolean done) {
                    callback.onDownloadProgress(bytesRead, contentLength, done);
                }
            };

            progressRequestListener = new ProgressRequestBody.ProgressRequestListener() {
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done) {
                    callback.onUploadProgress(bytesWritten, contentLength, done);
                }
            };
        }

        okhttp3.Call call = getTokenValidateBeforeCall(tokenAuthInfo, progressListener, progressRequestListener);
        Type localVarReturnType = new TypeToken<TokenAuthResponse>(){}.getType();
        apiClient.executeAsync(call, localVarReturnType, callback);
        return call;
    }
}
