package com.huawei.dli.sdk;

import com.huawei.dli.restapi.ApiException;
import com.huawei.dli.sdk.exception.DLIException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import lombok.Getter;

@Getter
public class Cluster {
    private static final Logger LOGGER = LoggerFactory.getLogger(Cluster.class);

    private String clusterName;

    private String owner;

    private int cuCount;

    private String description;

    private long createTime;

    private String status;

    private DLIClient client;

    public Cluster(String clusterName, String owner, int cuCount, String description, long createTime,
        String status, DLIClient client) {
        this.clusterName = clusterName;
        this.owner = owner;
        this.cuCount = cuCount;
        this.description = description;
        this.createTime = createTime;
        this.status = status;
        this.client = client;
    }

    /**
     * Delete this cluster.
     *
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public void deleteCluster() throws DLIException {
        try {
            // On Spark serverless, when delete success, it return nothing. This is different from SQL.
            client.getRestApi().deleteCluster(client.getProjectId(), clusterName, client.getAuthToken());
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    @Override
    public String toString() {
        return "Cluster{"
            + "clusterName='" + clusterName + '\''
            + ", owner='" + owner + '\''
            + ", cuCount='" + cuCount
            + ", description='" + description + '\''
            + ", createTime=" + createTime
            + ", status='" + status + '\''
            + "}";
    }
}
