package com.huawei.dli.sdk;

import com.huawei.dli.restapi.ApiException;
import com.huawei.dli.restapi.api.RestApi;
import com.huawei.dli.restapi.model.ClusterInfo;
import com.huawei.dli.restapi.model.ClusterInfos;
import com.huawei.dli.restapi.model.CreateClusterRequest;
import com.huawei.dli.restapi.model.CreateFlinkJarJobRequest;
import com.huawei.dli.restapi.model.CreateFlinkJobTemplateRequest;
import com.huawei.dli.restapi.model.CreateTsdbReq;
import com.huawei.dli.restapi.model.CreateTsdbResp;
import com.huawei.dli.restapi.model.DatabaseInfo;
import com.huawei.dli.restapi.model.DatabaseResult;
import com.huawei.dli.restapi.model.DeleteJobInBatchRequest;
import com.huawei.dli.restapi.model.FlinkJobApigSinksResponse;
import com.huawei.dli.restapi.model.FlinkJobExecutePlanResponse;
import com.huawei.dli.restapi.model.FlinkJobTemplateCreateResponse;
import com.huawei.dli.restapi.model.FlinkJobTemplateDeleteResponse;
import com.huawei.dli.restapi.model.FlinkJobTemplateListResponse;
import com.huawei.dli.restapi.model.GetFlinkJobDetailResponse;
import com.huawei.dli.restapi.model.GetFlinkJobsMetricsBody;
import com.huawei.dli.restapi.model.GetTsdbByIdResp;
import com.huawei.dli.restapi.model.GetTsdbListResp;
import com.huawei.dli.restapi.model.GlobalBatchResponse;
import com.huawei.dli.restapi.model.GlobalResponse;
import com.huawei.dli.restapi.model.GroupResourceInfo;
import com.huawei.dli.restapi.model.JobListInfo;
import com.huawei.dli.restapi.model.JobListResult;
import com.huawei.dli.restapi.model.JobProgressListInfo;
import com.huawei.dli.restapi.model.JobProgressResult;
import com.huawei.dli.restapi.model.JobStatusResponse;
import com.huawei.dli.restapi.model.JobUpdateResponse;
import com.huawei.dli.restapi.model.MessageInfo;
import com.huawei.dli.restapi.model.ModuleResourceInfo;
import com.huawei.dli.restapi.model.ObsBuckets;
import com.huawei.dli.restapi.model.PartitionInfo;
import com.huawei.dli.restapi.model.PartitionListInfo;
import com.huawei.dli.restapi.model.PartitionResult;
import com.huawei.dli.restapi.model.PkgResourceInfo;
import com.huawei.dli.restapi.model.PkgResourceRequest;
import com.huawei.dli.restapi.model.QueryFlinkJobListResponse;
import com.huawei.dli.restapi.model.QueryFlinkJobMetricsResponse;
import com.huawei.dli.restapi.model.QueueActionRequest;
import com.huawei.dli.restapi.model.QueueActionResponse;
import com.huawei.dli.restapi.model.QueueInfo;
import com.huawei.dli.restapi.model.QueueResult;
import com.huawei.dli.restapi.model.ResourceInfos;
import com.huawei.dli.restapi.model.ResourceMsg;
import com.huawei.dli.restapi.model.ResponseInfo;
import com.huawei.dli.restapi.model.ResponseJobInfo;
import com.huawei.dli.restapi.model.ResponseQueueInfo;
import com.huawei.dli.restapi.model.RunFlinkJobRequest;
import com.huawei.dli.restapi.model.SQLJobRequest;
import com.huawei.dli.restapi.model.SessionInfo;
import com.huawei.dli.restapi.model.SessionInfos;
import com.huawei.dli.restapi.model.StatusResult;
import com.huawei.dli.restapi.model.StopFlinkJobRequest;
import com.huawei.dli.restapi.model.SubmitFlinkSqlJobRequest;
import com.huawei.dli.restapi.model.UpdateFlinkJarJobRequest;
import com.huawei.dli.restapi.model.UpdateFlinkJobTemplateRequest;
import com.huawei.dli.restapi.model.UpdateFlinkSqlJobRequest;
import com.huawei.dli.restapi.model.UpdateResourceOwner;
import com.huawei.dli.restapi.model.UpdateTsdbReq;
import com.huawei.dli.sdk.authentication.Authentication;
import com.huawei.dli.sdk.authentication.AuthenticationMode;
import com.huawei.dli.sdk.common.BatchListFilter;
import com.huawei.dli.sdk.common.ChargingMode;
import com.huawei.dli.sdk.common.DLIInfo;
import com.huawei.dli.sdk.common.JobFilter;
import com.huawei.dli.sdk.common.JobProgressInfo;
import com.huawei.dli.sdk.common.JobProgressResultInfo;
import com.huawei.dli.sdk.common.JobResultInfo;
import com.huawei.dli.sdk.common.JobType;
import com.huawei.dli.sdk.common.SparkJobResultInfo;
import com.huawei.dli.sdk.exception.DLIException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import okhttp3.OkHttpClient;

public class DLIClient {

    private static final int MAX_PAGE_SIZE = 100;

    private static final Logger LOGGER = LoggerFactory.getLogger(DLIClient.class);

    private final DLIInfo info;

    private final Authentication auth;

    public DLIClient(AuthenticationMode mode, DLIInfo info) {
        this.info = info;
        auth = new Authentication(mode, info);
    }

    public void setAuthToken(String authToken) {
        auth.setAuthToken(authToken);
    }

    public void setAccessKey(String accessKey) {
        auth.setAccessKey(accessKey);
    }

    public void setSecretKey(String secretKey) {
        auth.setSecretKey(secretKey);
    }

    public void setSecurityToken(String securityToken) {
        auth.setSecurityToken(securityToken);
    }

    public Authentication getAuthentication() {
        return auth;
    }

    public DLIInfo getDLIInfo() {
        return info;
    }

    public RestApi getRestApi() {
        return auth.getRestApi();
    }

    public String getProjectId() {
        return auth.getProjectId();
    }

    public String getAuthToken() throws DLIException {
        return auth.getAuthToken();
    }

    /**
     * Set the default connect timeout for new connections.
     *
     * @param connectionTimeout Unit:SECONDS
     */
    public void setConnectTimeout(int connectionTimeout) {
        OkHttpClient newHttpClient =  auth.getRestApi().getApiClient().getHttpClient().newBuilder()
            .connectTimeout(connectionTimeout, TimeUnit.SECONDS).build();
        auth.getRestApi().getApiClient().setHttpClient(newHttpClient);
    }

    /**
     * Set the default read timeout for new connections.
     *
     * @param readTimeout Unit:SECONDS
     */
    public void setReadTimeout(int readTimeout) {
        OkHttpClient newHttpClient =  auth.getRestApi().getApiClient().getHttpClient().newBuilder()
            .readTimeout(readTimeout, TimeUnit.SECONDS).build();
        auth.getRestApi().getApiClient().setHttpClient(newHttpClient);
    }

    /**
     * Set proxy for new connections.
     *
     * @param proxyHost proxy ip
     * @param proxyPort proxy port
     */
    public void setProxy(String proxyHost, int proxyPort) {
        auth.setProxy(proxyHost, proxyPort);
    }

    /**
     * Set httpClient for new connections.
     *
     * @param httpClient http client
     */
    public void setHttpClient(OkHttpClient httpClient) {
        auth.setHttpClient(httpClient);
    }

    /**
     * 队列操作
     *
     * @param queueName queue名称，指定queue (required)
     * @param body      队列操作请求体, 请求参数为json格式 (required)
     * @return QueueActionResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public QueueActionResponse queueAction(String queueName, QueueActionRequest body) throws DLIException {
        try {
            return getRestApi().queueAction(getProjectId(), queueName, body, getAuthToken());
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 用户主动授权起OBS桶的操作权限给DLI服务, 用于保存用户作业的checkpoint、作业的运行日志等
     *
     * @param obsBuckets 授权一个或多个OBS bucket权限给Data lake insight, 请求参数为json格式 (required)
     * @return GlobalResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public GlobalResponse authorizeBucket(ObsBuckets obsBuckets) throws DLIException {
        try {
            return getRestApi().authorizeBucket(getProjectId(), obsBuckets, getAuthToken());
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 提交流式SQL作业到DLI服务
     * 通过POST方式, 提交流式SQL作业, 请求体为JSON格式
     *
     * @param body 提交创建SQL作业请求 (required)
     * @return JobStatusResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public JobStatusResponse submitFlinkSqlJob(SubmitFlinkSqlJobRequest body) throws DLIException {
        try {
            JobStatusResponse response = getRestApi().submitFlinkSqlJob(getProjectId(), body, getAuthToken());
            LOGGER.info("FlinkSqlJob submit succeed, jobId: " + response.getJob().getJobId());
            return response;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 更新流式SQL作业
     * 流式SQL的语法扩展了Apache Flink SQL, 具体详情请参考DLI官方文档
     *
     * @param body  提交修改SQL作业请求 (required)
     * @param jobId 作业ID (required)
     * @return JobUpdateResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public JobUpdateResponse updateFlinkSqlJob(UpdateFlinkSqlJobRequest body, Long jobId) throws DLIException {
        try {
            JobUpdateResponse response = getRestApi().updateFlinkSqlJob(getProjectId(), jobId, body, getAuthToken());
            LOGGER.info("FlinkSqlJob update succeed, jobId: " + jobId);
            return response;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 创建一个用户自定义Flink作业
     * 用户自定义作业目前支持jar格式, 运行在独享集群中
     *
     * @param body 提交创建自定义Flink作业请求 (required)
     * @return JobStatusResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public JobStatusResponse createFlinkJarJob(CreateFlinkJarJobRequest body) throws DLIException {
        try {
            JobStatusResponse response = getRestApi().submitFlinkJarJob(getProjectId(), body, getAuthToken());
            LOGGER.info("Create flinkJarJob succeed, jobId: " + response.getJob().getJobId());
            return response;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 更新用户自定义Flink作业
     * 目前仅支持Jar格式, 运行在独享集群中
     *
     * @param body  提交修改自定义Flink作业请求 (required)
     * @param jobId 作业ID (required)
     * @return JobUpdateResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public JobUpdateResponse updateFlinkJarJob(UpdateFlinkJarJobRequest body, Long jobId) throws DLIException {
        try {
            JobUpdateResponse response = getRestApi().updateFlinkJarJob(getProjectId(), jobId, body, getAuthToken());
            LOGGER.info("FlinkJarJob update succeed , jobId: " + jobId);
            return response;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 查询作业监控信息
     * 查询作业监控信息, 支持同时查询多个作业监控信息
     *
     * @param body 查询作业监控信息, 支持同时查询多个作业监控, 请求参数为json格式 (required)
     * @return QueryFlinkJobMetricsResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public QueryFlinkJobMetricsResponse getFlinkJobsMetrics(GetFlinkJobsMetricsBody body) throws DLIException {
        try {
            return getRestApi().getFlinkJobsMetrics(getProjectId(), body, getAuthToken());
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 查询作业列表
     * 作业列表查询, 支持以下参数: name, status, show_detail, offset, limit, order, root_job_id.
     *
     * @param name        作业名 (optional)
     * @param status      作业状态码, 请参考DLI文档 (optional)
     * @param queueName   队列名称 (optional)
     * @param showDetail  是否返回作业详情信息 (optional, default to false)
     * @param offset      作业偏移量 (optional)
     * @param limit       返回的数据条数 (optional, default to 20)
     * @param order       查询结果排序, 升序和降序两种可选 (optional, default to desc)
     * @param rootJobId   边缘父作业ID, 用于查询指定边缘作业的子作业; 不带该参数时, 查询所有非边缘作业和边缘父作业,
     *                    不包括边缘子作业 (optional)
     * @param username    执行作业的用户名
     * @param jobType     作业类型
     * @param showJobType 显示作业类型
     * @return QueryFlinkJobListResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    @SuppressWarnings("checkstyle:ParameterNumber")
    public QueryFlinkJobListResponse getFlinkJobs(String name, String status, String queueName, Boolean showDetail,
        Long offset, Integer limit, String order, Long rootJobId, String username, String jobType, String showJobType)
        throws DLIException {
        try {
            return getRestApi().getFlinkJobs(getProjectId(), getAuthToken(), name, status,
                queueName, showDetail, offset, limit, order, rootJobId, username, jobType, showJobType);
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 查询作业详情
     * Get job detail information.
     *
     * @param jobId 作业ID (required)
     * @return GetFlinkJobDetailResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public GetFlinkJobDetailResponse getFlinkJobDetail(Long jobId) throws DLIException {
        try {
            return getRestApi().getFlinkJobDetail(getProjectId(), jobId, getAuthToken());
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 删除任何状态的作业
     *
     * @param jobId 作业ID (required)
     * @return GlobalResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public GlobalResponse deleteJob(Long jobId) throws DLIException {
        try {
            GlobalResponse response = getRestApi().deleteJob(getProjectId(), jobId, getAuthToken());
            LOGGER.info("Job is deleted succeed, jobId: " + jobId);
            return response;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 批量删除作业
     *
     * @param body 作业ID数组 (required)
     * @return GlobalResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<GlobalBatchResponse> deleteFlinkJobInBatch(DeleteJobInBatchRequest body) throws DLIException {
        try {
            List<GlobalBatchResponse> response = getRestApi().deleteFlinkJob(getProjectId(), body, getAuthToken());
            LOGGER.info("FlinkJobInBatch delete succeed");
            return response;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 批量运行作业
     * 触发运行作业
     *
     * @param body 提交批量运行作业请求, 请求参数为json格式 (required)
     * @return GlobalResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<GlobalBatchResponse> runFlinkJob(RunFlinkJobRequest body) throws DLIException {
        try {
            List<GlobalBatchResponse> response = getRestApi().runFlinkJob(getProjectId(), body, getAuthToken());
            LOGGER.info("FlinkJobs run succeed");
            return response;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 批量停止作业
     *
     * @param body 提交批量停止作业请求, 请求参数为json格式 (required)
     * @return GlobalResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<GlobalBatchResponse> stopFlinkJob(StopFlinkJobRequest body) throws DLIException {
        try {
            List<GlobalBatchResponse> response = getRestApi().stopFlinkJob(getProjectId(), body, getAuthToken());
            LOGGER.info("FlinkJob stop succeed");
            return response;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 取消flink自定义作业
     *
     * @param jobId             取消flink自定义作业id (required)
     * @param isResumeSavepoint 是否恢复savepoint
     * @return GlobalResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public GlobalBatchResponse stopSingleFlinkJob(Long jobId, Boolean isResumeSavepoint) throws DLIException {
        try {
            StopFlinkJobRequest body = new StopFlinkJobRequest();
            List<Long> jobIds = new ArrayList<>();
            jobIds.add(jobId);
            body.setJobIds(jobIds);
            body.setTriggerSavepoint(isResumeSavepoint);
            List<GlobalBatchResponse> response = getRestApi().stopFlinkJob(getProjectId(), body, getAuthToken());
            LOGGER.info("Single FlinkJob stop succeed, jobId: " + jobId);
            return response.get(0);
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 查询作业执行计划图
     * The execution graph is json format.
     *
     * @param jobId 作业ID (required)
     * @return FlinkJobExecutePlanResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public FlinkJobExecutePlanResponse getFlinkJobExecuteGraph(Long jobId) throws DLIException {
        try {
            return getRestApi().getFlinkJobExecuteGraph(getProjectId(), jobId, getAuthToken());
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 查询作业apig网关服务访问地址
     * The apig sinks is json format.
     *
     * @param jobId 作业ID (required)
     * @return FlinkJobApigSinksResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public FlinkJobApigSinksResponse getFlinkApigSinks(Long jobId) throws DLIException {
        try {
            return getRestApi().getFlinkApigSinks(getProjectId(), jobId, getAuthToken());
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 查询作业模板列表
     *
     * @param offset 作业偏移量 (optional)
     * @param limit  查询条数限制 (optional)
     * @param order  查询结果排序, 升序和降序两种可选 (optional, default to desc)
     * @return FlinkJobTemplateListResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public FlinkJobTemplateListResponse getFlinkJobTemplates(Long offset, Integer limit, String order)
        throws DLIException {
        try {
            return getRestApi().getFlinkJobTemplates(getProjectId(), getAuthToken(),
                offset, limit, order);
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 删除作业模板
     * 当前模板被引用也允许删除模板
     *
     * @param templateId (required)
     * @return FlinkJobTemplateDeleteResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public FlinkJobTemplateDeleteResponse deleteFlinkJobTemplate(Long templateId) throws DLIException {
        try {
            FlinkJobTemplateDeleteResponse response = getRestApi().deleteFlinkJobTemplate(getProjectId(),
                templateId, getAuthToken());
            LOGGER.info("JobTemplate delete succeed, templateId: " + templateId);
            return response;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 创建作业模板
     * Create job template on Data lake insight
     *
     * @param body Create job template request (required)
     * @return FlinkJobTemplateCreateResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public FlinkJobTemplateCreateResponse createFlinkJobTemplate(CreateFlinkJobTemplateRequest body)
        throws DLIException {
        try {
            FlinkJobTemplateCreateResponse response = getRestApi().createFlinkJobTemplate(getProjectId(),
                body, getAuthToken());
            LOGGER.info("Create flinkJobTemplate succeed, jobTemplateId: " + response.getTemplate().getTemplateId());
            return response;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * 更新作业模板
     * Update job template on Data lake insight
     *
     * @param body       提交修改SQL作业模板请求 (required)
     * @param templateId 作业模板ID
     * @return GlobalResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public GlobalResponse updateFlinkJobTemplate(UpdateFlinkJobTemplateRequest body, Long templateId)
        throws DLIException {
        try {
            GlobalResponse response = getRestApi().updateFlinkJobTemplate(getProjectId(), templateId, body,
                getAuthToken());
            LOGGER.info("FlinkJobTemplate update succeed, templateId: " + templateId);
            return response;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Lists the currently user-owned Queues.
     * Include information:queueName,owner,cu,description and createTime.
     *
     * @return List Queues
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<Queue> listAllQueues() throws DLIException {
        List<Queue> queues = new ArrayList<>();
        try {
            QueueResult result = getRestApi().listAllQueues(getProjectId(), getAuthToken(), "all");
            if (result.getIsSuccess()) {
                List<QueueInfo> queueList = result.getQueues();
                for (QueueInfo queueInfo : queueList) {
                    Queue queue = new Queue(queueInfo.getQueueId(), queueInfo.getQueueName(), queueInfo.getOwner(),
                        queueInfo.getCuCount(), queueInfo.getDescription(), queueInfo.getCreateTime(),
                        ChargingMode.fromValue(queueInfo.getChargingMode()), this);
                    queues.add(queue);
                }
                return queues;
            }
            LOGGER.error(result.getMessage());
            throw new DLIException(result.getMessage());
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Get default queue,
     * get a Queue object and queueName is "default"
     *
     * @return default Queue
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public Queue getDefaultQueue() throws DLIException {
        return getQueue("default");
    }

    /**
     * Get a Queue object by queueName.Must be sure that the name exists,
     * otherwise throw exception:"There is no Queue named"
     *
     * @param queueName queue name
     * @return Queue
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public Queue getQueue(String queueName) throws DLIException {
        List<Queue> queues = listAllQueues();
        for (Queue queue : queues) {
            if (queue.getQueueName().equalsIgnoreCase(queueName)) {
                return queue;
            }
        }
        LOGGER.error("There is no Queue named " + queueName);
        throw new DLIException("There is no Queue named " + queueName);
    }

    /**
     * Create a queue.
     *
     * @param queueName    The name of the newly created resource queue.
     *                     Names can only contain numbers,
     *                     English letters, and underscores,
     *                     but they cannot be pure numbers,
     *                     and cannot begin with an underscore.
     * @param cuCount      The minimum number of calculation units bound to the queue.
     * @param chargingMode Queue charging model
     * @param description  The description of the queue.
     * @return Queue
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public Queue createQueue(String queueName, int cuCount,
        ChargingMode chargingMode, String description) throws DLIException {
        try {
            QueueInfo queueInfo = new QueueInfo();
            queueInfo.setQueueName(queueName);
            queueInfo.setCuCount(cuCount);
            queueInfo.setChargingMode(chargingMode.getValue());
            queueInfo.setDescription(description);
            ResponseQueueInfo response = getRestApi().createQueue(getProjectId(), queueInfo, getAuthToken());
            if (!response.getIsSuccess()) {
                LOGGER.error(response.getMessage());
                throw new DLIException(response.getMessage());
            }
            LOGGER.info("Create queue succeed, queueName: " + queueName);
            return getQueue(queueName);
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * @param databaseName database name
     * @return Database
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public Database createDatabase(String databaseName) throws DLIException {
        return createDatabase(databaseName, null);
    }

    /**
     * Create a database,must sure databaseName is not exist,
     * and there are enough quotas.
     * Description is not necessary,you can input null,or use "createDatabase(String databaseName)".
     *
     * @param databaseName database name
     * @param description  the description of database
     * @return database
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public Database createDatabase(String databaseName, String description) throws DLIException {
        DatabaseInfo databaseInfo = new DatabaseInfo();
        databaseInfo.setDatabaseName(databaseName);
        databaseInfo.setDescription(description);
        try {
            ResponseInfo response = getRestApi().createDatabase(getProjectId(), databaseInfo, getAuthToken());
            if (!response.getIsSuccess()) {
                LOGGER.error(response.getMessage());
                throw new DLIException(response.getMessage());
            }
            LOGGER.info("Create datebase succeed, databaseName: " + databaseName);
            return getDatabase(databaseName);
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    public Database createDatabase(
        String databaseName,
        String description,
        String enterpriseProjectId) throws DLIException {
        DatabaseInfo databaseInfo = new DatabaseInfo();
        databaseInfo.setDatabaseName(databaseName);
        databaseInfo.setDescription(description);
        databaseInfo.setEnterpriseProjectId(enterpriseProjectId);
        try {
            ResponseInfo response = getRestApi().createDatabase(getProjectId(), databaseInfo, getAuthToken());
            if (!response.getIsSuccess()) {
                LOGGER.error(response.getMessage());
                throw new DLIException(response.getMessage());
            }
            LOGGER.info("Create datebase succeed, databaseName: " + databaseName);
            return getDatabase(databaseName);
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    public Database getDatabase(String databaseName) throws DLIException {
        List<Database> databases = listAllDatabases();
        for (Database database : databases) {
            if (database.getDatabaseName().equalsIgnoreCase(databaseName)) {
                return database;
            }
        }
        LOGGER.error("There is no database named " + databaseName);
        throw new DLIException("There is no database named " + databaseName);
    }

    /**
     * List the currently user-owned all databases .
     * Databases information: databaseName,owner and description.
     *
     * @return List databases
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<Database> listAllDatabases() throws DLIException {
        List<Database> databases = new ArrayList<>();
        DatabaseResult result;
        try {
            result = getRestApi().listAllDatabases(getProjectId(), getAuthToken());
            if (result.getIsSuccess()) {
                List<DatabaseInfo> databaseList = result.getDatabases();
                for (DatabaseInfo dbInfo : databaseList) {
                    Database database = new Database(dbInfo.getDatabaseName(),
                        dbInfo.getDescription(), dbInfo.getOwner(), dbInfo.getTableNumber(), this);
                    if (dbInfo.getEnterpriseProjectId() != null) {
                        database.setEnterpriseProjectId(dbInfo.getEnterpriseProjectId());
                    }
                    if (dbInfo.getResourceId() != null) {
                        database.setResourceId(dbInfo.getResourceId());
                    }
                    databases.add(database);
                }
                return databases;
            }
            LOGGER.error(result.getMessage());
            throw new DLIException(result.getMessage());
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Obtains the job execution progress based on jobId.
     *
     * @param jobId the job id
     * @return JobProgressResultInfo
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public JobProgressResultInfo getJobProgress(String jobId) throws DLIException {
        try {
            JobProgressResult response = getRestApi().getJobProgress(getProjectId(), jobId, getAuthToken());
            if (!response.getIsSuccess()) {
                LOGGER.error(response.getMessage());
                throw new DLIException(response.getMessage());
            }

            List<JobProgressListInfo> jobListInfos = response.getSubJobs();
            List<JobProgressInfo> jobResultInfos = new ArrayList<>();
            if (jobListInfos != null) {
                for (JobProgressListInfo jobListInfo : jobListInfos) {
                    JobProgressInfo jobResultInfo = new JobProgressInfo(jobListInfo);
                    jobResultInfos.add(jobResultInfo);
                }
            }
            return new JobProgressResultInfo(
                response.getIsSuccess(), response.getStatus(), response.getSubJobId(), response.getProgress(),
                jobResultInfos);
        } catch (ApiException e) {
            LOGGER.error("jobId: {}, msg: {}", jobId, e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Get all JobsResultInfo list below the current project.
     * JobType default is ALL.
     *
     * @return List JobResultInfo
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<JobResultInfo> listAllJobs() throws DLIException {
        return this.listAllJobs(null, (Long) null, JobType.ALL);
    }

    /**
     * Get all JobsResultInfo list below the current project.
     * Filter by JobType.Such as:JobType.DDL
     *
     * @param jobType the job type enum value
     * @return Filtered JobResultInfo list
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<JobResultInfo> listAllJobs(JobType jobType) throws DLIException {
        return this.listAllJobs(null, (Long) null, jobType);
    }

    /**
     * Get all JobsResultInfo list below the current project.
     * Filter by JobType and start/end time. Such as:JobType.DDL
     * The time format is unix timestamp in milliseconds .
     *
     * @param start   Used to query jobs that start after this time.
     * @param end     Used to query jobs whose start time is before this point in time.
     * @param jobType Such as:JobType.DDL
     * @return Filtered JobResultInfo list
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<JobResultInfo> listAllJobs(Long start, Long end, JobType jobType) throws DLIException {
        List<JobResultInfo> jobResultInfos = new ArrayList<>();
        for (int i = 1; ; i++) {
            List<JobResultInfo> jobInfos = listAllJobs(MAX_PAGE_SIZE, i, start, end, jobType);
            jobResultInfos.addAll(jobInfos);
            if (jobInfos.size() < MAX_PAGE_SIZE) {
                break;
            }
        }
        return jobResultInfos;
    }

    /**
     * Get all JobsResultInfo list below the current project.
     * Filter by JobType and page.
     *
     * @param pageSize    The maximum number of jobs displayed on each page, in the range: [1, 100].
     * @param currentPage The current page number, the default is the first page.
     * @param jobType     Such as:JobType.DDL
     * @return Filtered JobResultInfo list
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<JobResultInfo> listAllJobs(Integer pageSize, Integer currentPage,
        JobType jobType) throws DLIException {
        return listAllJobs(pageSize, currentPage, null, null, jobType);
    }

    /**
     * Get all JobsResultInfo list below the current project.
     * Filter by JobType,page and start/end time.
     *
     * @param pageSize    The maximum number of jobs displayed on each page, in the range: [1, 100].
     * @param currentPage The current page number, the default is the first page.
     * @param start       Used to query jobs that start after this time.
     * @param end         Used to query jobs whose start time is before this point in time.
     * @param jobType     Such as:JobType.DDL
     * @return Filtered JobResultInfo list
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<JobResultInfo> listAllJobs(Integer pageSize, Integer currentPage, Long start,
        Long end, JobType jobType) throws DLIException {
        try {
            JobListResult response = getRestApi().listAllJobs(getProjectId(), getAuthToken(),
                pageSize, currentPage, start, end, jobType.name(), null, null, null, null, null);
            if (!response.getIsSuccess()) {
                LOGGER.error(response.getMessage());
                throw new DLIException(response.getMessage());
            }
            List<JobListInfo> jobListInfos = response.getJobs();
            List<JobResultInfo> jobResultInfos = new ArrayList<>();
            for (JobListInfo jobListInfo : jobListInfos) {
                JobResultInfo jobResultInfo = new JobResultInfo(jobListInfo);
                jobResultInfos.add(jobResultInfo);
            }
            return jobResultInfos;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Get all JobsResultInfo list below the current project.
     * Filter by JobType, start/end time and tags, queueName, owner, jobStatus, order
     *
     * @param jobFilter    the filter conditions.
     * @return Filtered JobResultInfo list
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<JobResultInfo> listAllJobs(JobFilter jobFilter) throws DLIException {
        try {
            String jobTypeName = JobType.ALL.name();
            if (jobFilter.getJobType() != null) {
                jobTypeName = jobFilter.getJobType().name();
            }
            List<JobResultInfo> jobResultInfos = new ArrayList<>();
            for (int i = 1; ; i++) {
                JobListResult response = getRestApi().listAllJobs(getProjectId(), getAuthToken(),
                    MAX_PAGE_SIZE, i, jobFilter.getStart(), jobFilter.getEnd(), jobTypeName, jobFilter.getTags(),
                        jobFilter.getQueueName(), jobFilter.getOwner(), jobFilter.getJobStatus(), jobFilter.getOrder());
                if (!response.getIsSuccess()) {
                    LOGGER.error(response.getMessage());
                    throw new DLIException(response.getMessage());
                }
                List<JobListInfo> jobListInfos = response.getJobs();
                for (JobListInfo jobListInfo : jobListInfos) {
                    JobResultInfo jobResultInfo = new JobResultInfo(jobListInfo);
                    jobResultInfos.add(jobResultInfo);
                }
                if (jobListInfos.size() < MAX_PAGE_SIZE) {
                    break;
                }
            }
            return jobResultInfos;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Get specify page and pageSize JobsResultInfo list below the current project.
     * Filter by JobType,page and start/end time and tags.
     *
     * @param jobFilter    the filter conditions.
     * @return Filtered JobResultInfo list
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<JobResultInfo> listJobsByPage(JobFilter jobFilter) throws DLIException {
        try {
            String jobTypeName = JobType.ALL.name();
            if (jobFilter.getJobType() != null) {
                jobTypeName = jobFilter.getJobType().name();
            }
            JobListResult response = getRestApi().listAllJobs(getProjectId(), getAuthToken(),
                jobFilter.getPageSize(), jobFilter.getCurrentPage(), jobFilter.getStart(),
                jobFilter.getEnd(), jobTypeName, jobFilter.getTags(), null, null, null, null);
            if (!response.getIsSuccess()) {
                LOGGER.error(response.getMessage());
                throw new DLIException(response.getMessage());
            }
            List<JobListInfo> jobListInfos = response.getJobs();
            List<JobResultInfo> jobResultInfos = new ArrayList<>();
            for (JobListInfo jobListInfo : jobListInfos) {
                JobResultInfo jobResultInfo = new JobResultInfo(jobListInfo);
                jobResultInfos.add(jobResultInfo);
            }
            return jobResultInfos;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Queue the JobResultInfo by job id.
     *
     * @param jobId Such as:"9e093e36-6efe-4549-8f04-db6f83c3c439".
     * @return The specific JobResultInfo queried by jobId.
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public JobResultInfo queryJobResultInfo(String jobId) throws DLIException {
        try {
            StatusResult response = getRestApi().getJobStatus(getProjectId(), jobId, getAuthToken());
            if (!response.getIsSuccess()) {
                LOGGER.error(response.getMessage());
                throw new DLIException(response.getMessage());
            }
            return new JobResultInfo(response);
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Query the information of the recently executed job submitted in the edit box under the current project.
     * Which the job submitted by SQLJob.
     *
     * @return List all SQLJob type jobResultInfo.
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<JobResultInfo> listSQLJobs() throws DLIException {
        List<JobResultInfo> jobResultInfos = new ArrayList<>();
        for (int i = 1; ; i++) {
            List<JobResultInfo> jobInfos = listSQLJobs(MAX_PAGE_SIZE, i);
            jobResultInfos.addAll(jobInfos);
            if (jobInfos.size() < MAX_PAGE_SIZE) {
                break;
            }
        }
        return jobResultInfos;
    }

    /**
     * Query the information of the recently executed job submitted in the edit box under the current project.
     * Which the job submitted by SQLJob.
     * Filter by page.
     *
     * @param pageSize    The maximum number of jobs displayed on each page, in the range: [1, 100].
     * @param currentPage The current page number, the default is the first page.
     * @return List all SQLJob type jobResultInfo.
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<JobResultInfo> listSQLJobs(Integer pageSize, Integer currentPage) throws DLIException {
        try {
            JobListResult response = getRestApi().listSqlJobs(getProjectId(), getAuthToken(),
                pageSize, currentPage, null);
            if (!response.getIsSuccess()) {
                LOGGER.error(response.getMessage());
                throw new DLIException(response.getMessage());
            }
            List<JobListInfo> jobListInfos = response.getJobs();
            List<JobResultInfo> jobResultInfos = new ArrayList<>();
            for (JobListInfo jobListInfo : jobListInfos) {
                JobResultInfo jobResultInfo = new JobResultInfo(jobListInfo);
                jobResultInfos.add(jobResultInfo);
            }
            return jobResultInfos;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Query all the information of the recently executed sql job submitted in the edit box under the current project.
     * Which the job submitted by SQLJob.
     * Filter by tags.
     *
     * @param jobFilter    the filter conditions, such as tags.
     * @return List all SQLJob type jobResultInfo.
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<JobResultInfo> listAllSQLJobs(JobFilter jobFilter) throws DLIException {
        try {
            List<JobResultInfo> jobResultInfos = new ArrayList<>();
            for (int i = 1; ; i++) {
                JobListResult response = getRestApi().listSqlJobs(getProjectId(), getAuthToken(),
                    MAX_PAGE_SIZE, i, jobFilter.getTags());
                if (!response.getIsSuccess()) {
                    LOGGER.error(response.getMessage());
                    throw new DLIException(response.getMessage());
                }
                List<JobListInfo> jobListInfos = response.getJobs();
                for (JobListInfo jobListInfo : jobListInfos) {
                    JobResultInfo jobResultInfo = new JobResultInfo(jobListInfo);
                    jobResultInfos.add(jobResultInfo);
                }
                if (jobListInfos.size() < MAX_PAGE_SIZE) {
                    break;
                }
            }
            return jobResultInfos;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Query the information of the recently executed job submitted in the edit box under the current project.
     * Which the job submitted by SQLJob.
     * Filter by page and tags.
     *
     * @param jobFilter    the filter conditions, such as pageSize, currentPage and tags.
     * @return List all SQLJob type jobResultInfo.
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<JobResultInfo> listSQLJobsByPage(JobFilter jobFilter) throws DLIException {
        try {
            JobListResult response = getRestApi().listSqlJobs(getProjectId(), getAuthToken(),
                jobFilter.getPageSize(), jobFilter.getCurrentPage(), jobFilter.getTags());
            if (!response.getIsSuccess()) {
                LOGGER.error(response.getMessage());
                throw new DLIException(response.getMessage());
            }
            List<JobListInfo> jobListInfos = response.getJobs();
            List<JobResultInfo> jobResultInfos = new ArrayList<>();
            for (JobListInfo jobListInfo : jobListInfos) {
                JobResultInfo jobResultInfo = new JobResultInfo(jobListInfo);
                jobResultInfos.add(jobResultInfo);
            }
            return jobResultInfos;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Cancel LAUNCHING,RUNNING status job,
     * and the job status changes to CANCELLED.
     *
     * @param jobId Such as:"9e093e36-6efe-4549-8f04-db6f83c3c439".
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public void cancelJob(String jobId) throws DLIException {
        if (jobId == null) {
            LOGGER.error("No job in progress,cancel failed");
            throw new DLIException("No job in progress,cancel failed");
        }
        try {
            ResponseInfo response = getRestApi().cancelJob(getProjectId(), jobId, getAuthToken());
            if (!response.getIsSuccess()) {
                LOGGER.error(response.getMessage());
                throw new DLIException(response.getMessage());
            }
            LOGGER.info("Cancel LAUNCHING,RUNNING status job succeed, jobId: " + jobId
                + ", and the job status changes to CANCELLED.");
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Check the syntax of SQL, if syntax is wrong, it will throw exception.
     *
     * @param sql The sql which should be checked.
     * @return ResponseJobInfo
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ResponseJobInfo checkSQL(String sql) throws DLIException {
        return checkSQL(sql, null);
    }

    /**
     * Check the syntax of SQL, if syntax is wrong, it will throw exception.
     *
     * @param sql       The sql which should be checked.
     * @param currentdb The database name of sql.
     * @return ResponseJobInfo
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public ResponseJobInfo checkSQL(String sql, String currentdb) throws DLIException {
        SQLJobRequest sqlJobRequest = new SQLJobRequest();
        if (null != currentdb) {
            sqlJobRequest.setCurrentdb(currentdb);
        }
        sqlJobRequest.setSql(sql);
        try {
            ResponseJobInfo responseJobInfo = getRestApi().checkSQL(getProjectId(), sqlJobRequest, getAuthToken());
            if (!responseJobInfo.getIsSuccess()) {
                LOGGER.error(responseJobInfo.getMessage());
                throw new DLIException(responseJobInfo.getMessage());
            }
            LOGGER.info("The syntax of SQL is right");
            return responseJobInfo;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    // Spark serverless related functions.

    /**
     * Create a cluster.
     *
     * @param clusterName Name of a new cluster. The name can contain only digits, letters, and underscores (_),
     *                    but cannot contain only digits or start with an underscore (_).
     * @param cuCount     Number of computing units (units to be billed) bound to a cluster.
     * @param description Description of a cluster.
     * @return Cluster
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public Cluster createCluster(String clusterName, int cuCount, String description) throws DLIException {
        try {
            CreateClusterRequest request = new CreateClusterRequest();
            request.setClusterName(clusterName);
            request.setCuCount(cuCount);
            request.setDescription(description);
            // The USER_ID is used for inner service to count it's real user, now we didn't use it.
            getRestApi().createCluster(getProjectId(), getAuthToken(), request, null);
            LOGGER.info("Create cluster succeed, clusterName: " + clusterName);
            return getCluster(clusterName);
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Get a Cluster object by queueName. Must be sure that the name exists,
     * otherwise throw exception: "There is no Queue named"
     *
     * @param clusterName Name of a new cluster. The name can contain only digits, letters, and underscores (_),
     *                    but cannot contain only digits or start with an underscore (_).
     * @return Cluster
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public Cluster getCluster(String clusterName) throws DLIException {
        // Here we can call rest directly, but now keep same logic as getQueue.
        List<Cluster> clusters = listAllClusters();
        for (Cluster cluster : clusters) {
            if (cluster.getClusterName().equalsIgnoreCase(clusterName)) {
                return cluster;
            }
        }
        LOGGER.error("There is no Cluster named " + clusterName);
        throw new DLIException("There is no Queue named " + clusterName);
    }

    /**
     * Lists the currently user-owned Clusters.
     * Include information: clusterName,owner,cu,description and createTime.
     *
     * @return list of Cluster
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<Cluster> listAllClusters() throws DLIException {
        List<Cluster> clusters = new ArrayList<>();
        try {
            ClusterInfos result = getRestApi().listCluster(getProjectId(), getAuthToken());
            if (result.getClusters() != null) {
                List<ClusterInfo> clusterInfos = result.getClusters();
                for (ClusterInfo clusterInfo : clusterInfos) {
                    Cluster cluster = new Cluster(clusterInfo.getClusterName(), clusterInfo.getOwner(),
                        clusterInfo.getCuCount(), clusterInfo.getDescription(), clusterInfo.getCreateTime(),
                        clusterInfo.getStatus(), this);
                    clusters.add(cluster);
                }
            }
            return clusters;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Upload resources async.
     *
     * @param kind        Resource type. Only support jar, pyFile, file.
     * @param paths       List of OBS object paths. The OBS object path refers to the OBS object URL.
     * @param description Package resource description.
     * @return resourceNames
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<String> uploadResourcesAsync(String kind, String[] paths, String description)
        throws DLIException {
        List<PackageResource> resources = uploadResources(true, kind, null, paths, description);
        return resources.stream().map(PackageResource::getResourceName).collect(Collectors.toList());
    }

    /**
     * Upload resources async.
     *
     * @param kind        Resource type. Only support jar, pyFile, file.
     * @param group       The name of the group to be created.
     * @param paths       List of OBS object paths. The OBS object path refers to the OBS object URL.
     * @param description Package resource description.
     * @return resourceNames
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<String> uploadResourcesAsync(String kind, String group, String[] paths, String description)
        throws DLIException {
        List<PackageResource> resources = uploadResources(true, kind, group, paths, description);
        return resources.stream().map(PackageResource::getResourceName).collect(Collectors.toList());
    }

    /**
     * Upload resources sync.
     *
     * @param kind        Resource type. Only support jar, pyFile, file.
     * @param paths       List of OBS object paths. The OBS object path refers to the OBS object URL.
     * @param description Package resource description.
     * @return list of PackageResource
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<PackageResource> uploadResources(String kind, String[] paths, String description)
        throws DLIException {
        return uploadResources(kind, null, paths, description);
    }

    /**
     * Upload resources sync.
     *
     * @param kind        Resource type. Only support jar, pyFile, file.
     * @param group       The name of the group to be created.
     * @param paths       List of OBS object paths. The OBS object path refers to the OBS object URL.
     * @param description Package resource description.
     * @return list of PackageResource
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<PackageResource> uploadResources(String kind, String group, String[] paths, String description)
        throws DLIException {
        return uploadResources(false, kind, group, paths, description);
    }

    /**
     * Upload resources(check async or sync).
     *
     * @param kind        Resource type. Only support jar, pyFile, file.
     * @param group       The name of the group to be created.
     * @param paths       List of OBS object paths. The OBS object path refers to the OBS object URL.
     * @param description Package resource description.
     * @return if async, just return resourceNames, else return whole resource info
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    private List<PackageResource> uploadResources(boolean isAsync, String kind,
        String group, String[] paths, String description)
        throws DLIException {
        List<PackageResource> packageResources = new ArrayList<>();
        try {
            PkgResourceRequest request = new PkgResourceRequest();
            request.setKind(kind);
            request.setGroup(group);
            request.setPaths(Arrays.asList(paths));
            request.setModule(null); // Common user can't upload module resources.
            request.setDescription(description);
            request.setIsAsync(isAsync);
            // The USER_ID is used for inner service to count it's real user, now we didn't use it.
            ResourceMsg response = getRestApi().uploadResources(getProjectId(), request, getAuthToken(), null);
            List<String> resourceNames = group == null ? response.getResourceNames() : response.getResources();
            if (response.getIsAsync() == null || !response.getIsAsync()) {
                for (String resourceName : resourceNames) {
                    packageResources.add(getResource(resourceName, group));
                }
            } else {
                resourceNames.forEach(i -> packageResources.add(new PackageResource(i)));
            }
            LOGGER.info("[isAsync={}]Resources upload succeed", isAsync);
            return packageResources;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Get a PackageResource object by resourceName. Must be sure that the name exists,
     * otherwise throw exception.
     *
     * @param resourceName Name of the resource package returned when the resource package is uploaded.
     * @return PackageResource
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public PackageResource getResource(String resourceName) throws DLIException {
        return getResource(resourceName, null);
    }

    /**
     * Get a PackageResource object by resourceName. Must be sure that the name exists,
     * otherwise throw exception.
     *
     * @param resourceName Name of the resource package returned when the resource package is uploaded.
     * @param group        The name of the resource group
     * @return PackageResource
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public PackageResource getResource(String resourceName, String group) throws DLIException {
        PackageResource packageResource;
        try {
            PkgResourceInfo result = getRestApi().getResourceInfo(getProjectId(), resourceName, group, getAuthToken());
            packageResource = new PackageResource(result);
            return packageResource;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Lists the currently user-owned Resources.
     * Include package resources and module resources.
     *
     * @return Resources
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public Resources listAllResources() throws DLIException {
        Resources resources = null;
        try {
            // Here we return all type resource.
            ResourceInfos result = getRestApi().listResources(getProjectId(), getAuthToken(), null);
            if (result != null) {
                List<PkgResourceInfo> pkgResourceInfos = result.getResources();
                PackageResource[] packageResources = new PackageResource[pkgResourceInfos.size()];
                for (int i = 0; i < pkgResourceInfos.size(); i++) {
                    packageResources[i] = new PackageResource(pkgResourceInfos.get(i));
                }
                List<ModuleResourceInfo> moduleResourceInfos = result.getModules();
                ModuleResource[] moduleResources = new ModuleResource[moduleResourceInfos.size()];
                for (int i = 0; i < moduleResourceInfos.size(); i++) {
                    moduleResources[i] = new ModuleResource(moduleResourceInfos.get(i));
                }
                List<GroupResourceInfo> groupResourceInfos = result.getGroups();
                GroupResource[] groupResources = new GroupResource[groupResourceInfos.size()];
                for (int i = 0; i < groupResourceInfos.size(); i++) {
                    groupResources[i] = new GroupResource(groupResourceInfos.get(i));
                }
                resources = new Resources(packageResources, moduleResources, groupResources, this);
            }
            return resources;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Delete package resources by name.
     *
     * @param resourceName name of the resource package
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public void deleteResource(String resourceName) throws DLIException {
        deleteResource(resourceName, null);
    }

    /**
     * Delete package resources by name and group.
     *
     * @param resourceName Name of the resource package
     * @param group        If group is not null,will delete the resource in the group
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public void deleteResource(String resourceName, String group) throws DLIException {
        try {
            getRestApi().deleteResource(getProjectId(), resourceName, group, getAuthToken());
            LOGGER.info("Resource is deleted succeed, resourceName: " + resourceName);
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Update resource or group owner
     *
     * @param updateResourceOwner new owner of the resource or group
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public void updateResourceOrGroupOwner(UpdateResourceOwner updateResourceOwner) throws DLIException {
        try {
            getRestApi().alertResourceOrGroupOwner(getProjectId(), updateResourceOwner, getAuthToken());
        } catch (ApiException e) {
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Lists the currently user-owned Spark batch jobs.
     *
     * @param clusterName Name of a cluster, which running the batch job.
     * @param from        Index number of the start batch processing job. By default, the index number starts from 0.
     * @param size        Number of batch processing jobs.
     * @return list of SparkJobResultInfo
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<SparkJobResultInfo> listAllBatchJobs(String clusterName, int from, int size) throws DLIException {
        List<SparkJobResultInfo> jobResultInfos = new ArrayList<>();
        try {
            SessionInfos sessionInfos = getRestApi().getBatchList(getProjectId(), getAuthToken(), clusterName,
                from, size, null, null, null, null, null);
            if (sessionInfos.getSessions() != null) {
                for (SessionInfo sessionInfo : sessionInfos.getSessions()) {
                    jobResultInfos.add(new SparkJobResultInfo(sessionInfo));
                }
            }
            return jobResultInfos;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * GET /v2.0/{project_id}/batches
     * Lists the currently user-owned Spark batch jobs.
     *
     * @param batchListFilter the filter conditions.
     * @return list of SparkJobResultInfo
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public List<SparkJobResultInfo> listAllBatchJobs(BatchListFilter batchListFilter) throws DLIException {
        List<SparkJobResultInfo> jobResultInfos = new ArrayList<>();
        try {
            SessionInfos sessionInfos = getRestApi().getBatchList(getProjectId(), getAuthToken(),
                batchListFilter.getClusterName(), batchListFilter.getFrom(), batchListFilter.getSize(),
                batchListFilter.getJobId(), batchListFilter.getJobName(), batchListFilter.getStartTime(),
                batchListFilter.getEndTime(), batchListFilter.getOrder());
            if (sessionInfos.getSessions() != null) {
                for (SessionInfo sessionInfo : sessionInfos.getSessions()) {
                    jobResultInfos.add(new SparkJobResultInfo(sessionInfo));
                }
            }
            return jobResultInfos;
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Delete batchJob by batchId
     *
     * @param batchId batch id
     * @return MessageInfo
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public MessageInfo delBatchJob(String batchId) throws DLIException {
        MessageInfo messageInfo;
        try {
            messageInfo = getRestApi().delBatch(getProjectId(), batchId, getAuthToken());
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
        LOGGER.info("BatchJob is deleted succeed, batchId: " + batchId);
        return messageInfo;
    }

    /**
     * Get describes the partition information for the table in specified offset and limit.
     *
     * @param databaseName database name
     * @param tableName    table name
     * @param offset       get offset
     * @param limit        get limit
     * @return PartitionResult
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public PartitionResult showPartitions(String databaseName, String tableName, Integer offset, Integer limit)
        throws DLIException {
        PartitionResult partitionResult;
        List<PartitionInfo> finalPartInfos = new ArrayList<>();
        try {
            if (offset < 0) {
                throw new DLIException("invalid offset when show partitions: " + offset);
            }
            if (limit < 0) {
                throw new DLIException("invalid limit when show partitions: " + limit);
            }

            // 此处取limit=1条数据是为了获取总的分区数，具体的分区信息通过多线程批量处理
            partitionResult = getRestApi().showPartitions(getProjectId(), databaseName, tableName, getAuthToken(),
                offset, 1);
            if (partitionResult.getIsSuccess()) {
                // 总的分区数
                Integer totalCount = partitionResult.getPartitions().getTotalCount();
                int startIndex = Math.min(totalCount, offset);
                int endIndex = Math.min(totalCount, offset + limit);
                int totalSizeToFetch = endIndex - startIndex;
                if (totalSizeToFetch > 5000) {
                    // 每个线程处理的总数据量
                    int threadSize = 5000;
                    // 线程数
                    int threadNum = (int) Math.ceil((double) totalSizeToFetch / (double) threadSize);
                    int threadPoolNum = Math.min(threadNum, info.getPartThreadNum());
                    LOGGER.info("get partition by " + threadPoolNum + " thread");
                    ExecutorService exec = Executors.newFixedThreadPool(threadPoolNum);
                    CountDownLatch latch = new CountDownLatch(threadNum);
                    Map<Integer, List<PartitionInfo>> partInfosMap = new HashMap<>();
                    Map<Integer, List<String>> msgMap = new HashMap<>();
                    for (int i = 0; i < threadNum; i++) {
                        List<PartitionInfo> taskPartInfos = new ArrayList<>();
                        partInfosMap.put(i, taskPartInfos);
                        List<String> message = new ArrayList<>();
                        msgMap.put(i, message);
                        int remain = endIndex - (offset + i * threadSize);
                        if (remain < 0) {
                            throw new DLIException("invalid offset: " + offset);
                        }
                        GetPartitionTask task = new GetPartitionTask(databaseName, tableName, offset + i * threadSize,
                            Math.min(remain, threadSize), taskPartInfos, this, message, latch);
                        try {
                            exec.submit(task);
                        } catch (Exception e) {
                            throw new RuntimeException("submit task error", e);
                        }
                    }
                    try {
                        latch.await();
                    } catch (InterruptedException e) {
                        throw new DLIException(e);
                    }
                    exec.shutdown();
                    for (Map.Entry<Integer, List<String>> msgEntry : msgMap.entrySet()) {
                        if (msgEntry.getValue() != null && msgEntry.getValue().size() > 0) {
                            StringBuilder sb = new StringBuilder();
                            for (String str : msgEntry.getValue()) {
                                sb.append(str);
                            }
                            throw new DLIException(sb.toString());
                        }
                    }
                    for (int i = 0; i < threadNum; i++) {
                        finalPartInfos.addAll(partInfosMap.get(i));
                    }
                } else {
                    LOGGER.info("get partition by single thread");
                    for (int start = offset; start < offset + limit; start += 500) {
                        int remain = offset + limit - start;
                        partitionResult = getRestApi().showPartitions(getProjectId(), databaseName, tableName,
                            getAuthToken(), start, Math.min(remain, 500));
                        if (partitionResult.getIsSuccess()) {
                            finalPartInfos.addAll(partitionResult.getPartitions().getPartitionInfos());
                        } else {
                            throw new DLIException(partitionResult.getMessage());
                        }
                    }
                }
                PartitionListInfo partitionListInfo = new PartitionListInfo();
                partitionListInfo.setPartitionInfos(finalPartInfos);
                partitionListInfo.setTotalCount(totalCount);
                partitionResult.setPartitions(partitionListInfo);
                return partitionResult;
            }
            LOGGER.error("this table have no partitions");
            throw new DLIException("this table have no partitions");
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    /**
     * Get describes the partition information for the specified table
     *
     * @param databaseName database name
     * @param tableName    table name
     * @return PartitionResult
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public PartitionResult showPartitions(String databaseName, String tableName) throws DLIException {
        PartitionResult partitionResult;
        try {
            // 此处取limit=1条数据是为了获取总的分区数，具体的分区信息通过多线程批量处理
            partitionResult = getRestApi().showPartitions(getProjectId(), databaseName, tableName, getAuthToken(),
                0, 1);
            if (partitionResult.getIsSuccess()) {
                List<PartitionInfo> partitionInfos = this.showPartitions(databaseName, tableName, 0,
                    partitionResult.getPartitions().getTotalCount()).getPartitions().getPartitionInfos();
                PartitionListInfo partitions = partitionResult.getPartitions();
                partitions.setPartitionInfos(partitionInfos);
                partitionResult.setPartitions(partitions);
            }
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
        return partitionResult;
    }


    /**
     * 创建时序数据库配置信息
     *
     * @param createTsdbReq This is a auto create Body Object (required)
     * @param projectId  (required)
     * @return CreateTsdbResp
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public CreateTsdbResp createTsdb(CreateTsdbReq createTsdbReq, String projectId) throws DLIException {
        CreateTsdbResp createTsdbResp;
        try {
            createTsdbResp = getRestApi().createTsdb(createTsdbReq, projectId, getAuthToken());
        } catch (ApiException e) {
            LOGGER.error("Failed to create tsdb.", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
        LOGGER.info("Success to create tsdb, projectId:" + projectId);
        return createTsdbResp;
    }

    /**
     * 删除时序数据库配置信息
     * 删除时序数据库配置信息
     * @param projectId  (required)
     * @param tsdbId  (required)
     * @return GlobalResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public GlobalResponse deleteTsdb(String projectId, String tsdbId) throws DLIException {
        GlobalResponse globalResponse;
        try {
            globalResponse = getRestApi().deleteTsdb(projectId, tsdbId, getAuthToken());
        } catch (ApiException e) {
            LOGGER.error("Failed to delete tsdb.", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
        LOGGER.info("Success to delete tsdb, tsdbId:" + tsdbId + " projectId:" + projectId);
        return globalResponse;
    }


    /**
     * 查询时序数据库配置信息详情
     * 根据时序数据库ID，查询时序数据库详情
     * @param projectId 项目ID (required)
     * @param tsdbId 时序数据库的ID (required)
     * @return GetTsdbByIdResp
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public GetTsdbByIdResp getTsdbById(String projectId, Long tsdbId) throws DLIException {
        GetTsdbByIdResp getTsdbByIdResp;
        try {
            getTsdbByIdResp = getRestApi().getTsdbById(projectId, tsdbId, getAuthToken());
        } catch (ApiException e) {
            LOGGER.error("Failed to get tsdb by id.", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
        LOGGER.info("Success to get tsdb by id, tsdbId:" + tsdbId + " projectId:" + projectId);
        return getTsdbByIdResp;
    }


    /**
     * 查询时序数据库配置信息列表
     *
     * @param projectId  (required)
     * @param offset 偏移量 (optional, default to 0)
     * @param limit 每页的数量 (optional, default to 10)
     * @return GetTsdbListResp
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public GetTsdbListResp getTsdbList(String projectId, Integer offset, Integer limit) throws DLIException {
        GetTsdbListResp getTsdbListResp;
        try {
            getTsdbListResp = getRestApi().getTsdbList(projectId, offset, limit, getAuthToken());
        } catch (ApiException e) {
            LOGGER.error("Failed to get tsdb list.", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
        LOGGER.info("Success to get tsdb list, projectId:" + projectId + " offset:" + offset + " limit:" + limit);
        return getTsdbListResp;
    }

    /**
     * 修改时序数据库配置信息
     * 修改时序数据库配置信息
     * @param projectId  (required)
     * @param tsdbId  (required)
     * @param updateTsdbReq This is a auto create Body Object (optional)
     * @return GlobalResponse
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public GlobalResponse updateTsdb(String projectId, String tsdbId, UpdateTsdbReq updateTsdbReq) throws DLIException {
        GlobalResponse globalResponse;
        try {
            globalResponse = getRestApi().updateTsdb(projectId, tsdbId, updateTsdbReq, getAuthToken());
        } catch (ApiException e) {
            LOGGER.error("Failed to update tsdb.", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
        LOGGER.info("Success to update tsdb, projectId:" + projectId + " tsdbId:" + tsdbId);
        return globalResponse;
    }
}
