package com.huawei.dli.sdk;

import com.huawei.dli.restapi.ApiException;
import com.huawei.dli.restapi.model.ExportJobRequest;
import com.huawei.dli.restapi.model.ResponseJobInfo;
import com.huawei.dli.sdk.common.CompressType;
import com.huawei.dli.sdk.common.ExportMode;
import com.huawei.dli.sdk.common.StorageType;
import com.huawei.dli.sdk.exception.DLIException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import lombok.Getter;

@Getter
public class ExportJob extends Job {

    private static final StorageType STORAGE_TYPE_DEFAULT = StorageType.CSV;

    private static final CompressType COMPRESS_TYPE_DEFAULT = CompressType.GZIP;

    private static final ExportMode EXPORT_MODE_DEFAULT = ExportMode.ERRORIFEXISTS;

    private static final Logger LOGGER = LoggerFactory.getLogger(ExportJob.class);

    private final String databaseName;

    private final String tableName;

    private final String dataPath;

    private StorageType storageType;

    private CompressType compressType;

    private ExportMode exportMode;

    public ExportJob(Queue queue, String databaseName, String tableName, String dataPath) {
        super(queue);
        this.databaseName = databaseName;
        this.tableName = tableName;
        this.dataPath = dataPath;
        this.storageType = STORAGE_TYPE_DEFAULT;
        this.compressType = COMPRESS_TYPE_DEFAULT;
        this.exportMode = EXPORT_MODE_DEFAULT;
    }

    public void setStorageType(StorageType storageType) {
        this.storageType = storageType;
    }

    public void setCompressType(CompressType compressType) {
        this.compressType = compressType;
    }

    public void setExportMode(ExportMode exportMode) {
        this.exportMode = exportMode;
    }

    /**
     * Submit export job.
     *
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public void submit() throws DLIException {
        if (this.getJobId() != null) {
            throw new DLIException("Job had been submitted,don't submit repeatedly");
        }
        ExportJobRequest jobRequest = new ExportJobRequest();
        jobRequest.setDataPath(dataPath);
        jobRequest.setDatabaseName(databaseName);
        jobRequest.setTableName(tableName);
        jobRequest.setDataType(storageType.getName());
        jobRequest.setCompress(compressType.getName());
        jobRequest.setExportMode(exportMode.getName());
        if (!"default".equals(getQueue().getQueueName())) {
            jobRequest.setQueueName(getQueue().getQueueName());
        }
        ResponseJobInfo response = null;
        try {
            response = client.getRestApi().exportData(getProjectId(), jobRequest, client.getAuthToken());
            if (!response.getIsSuccess()) {
                LOGGER.error(response.getMessage());
                throw new DLIException(response.getMessage());
            }
            super.setJobId(response.getJobId());
            super.checkJobRunningStatus(response.getJobId());
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }
}
