package com.huawei.dli.sdk;

import com.huawei.dli.restapi.ApiException;
import com.huawei.dli.restapi.model.PartitionInfo;
import com.huawei.dli.restapi.model.PartitionResult;
import com.huawei.dli.sdk.exception.DLIException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.concurrent.CountDownLatch;

public class GetPartitionTask implements Runnable {
    private static final Logger LOGGER = LoggerFactory.getLogger(GetPartitionTask.class);

    private final String dbName;

    private final String tblName;

    private final int offset;

    private final int limit;

    private final List<PartitionInfo> partitionInfos;

    private final DLIClient client;

    private final List<String> message;

    private final CountDownLatch cdl;

    public GetPartitionTask(String dbName, String tblName, int offset, int limit, List<PartitionInfo> partitionInfos,
        DLIClient client, List<String> message, CountDownLatch cdl) {
        this.dbName = dbName;
        this.tblName = tblName;
        this.offset = offset;
        this.limit = limit;
        this.partitionInfos = partitionInfos;
        this.client = client;
        this.message = message;
        this.cdl = cdl;
    }

    @Override
    public void run() {
        try {
            LOGGER.info("starting get partition with offset: " + offset);
            for (int start = offset; start < offset + limit; start += 500) {
                int remain = offset + limit - start;
                PartitionResult partitionResult = client.getRestApi()
                    .showPartitions(client.getProjectId(), dbName, tblName,
                        client.getAuthToken(), start, Math.min(remain, 500));
                if (partitionResult.getIsSuccess()) {
                    partitionInfos.addAll(partitionResult.getPartitions().getPartitionInfos());
                } else {
                    String msg = partitionResult.getMessage();
                    LOGGER.error("get partition failed, " + msg);
                    message.add(msg);
                    throw new DLIException(msg);
                }
            }
        } catch (DLIException | ApiException e) {
            LOGGER.error("get partition cause exception", e);
            message.add(e.getMessage());
        } finally {
            cdl.countDown();
        }
    }
}
