package com.huawei.dli.sdk;

import com.huawei.dli.restapi.ApiException;
import com.huawei.dli.restapi.model.ResponseInfo;
import com.huawei.dli.sdk.common.ChargingMode;
import com.huawei.dli.sdk.exception.DLIException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import lombok.Getter;

@Getter
public class Queue {

    private static final Logger LOGGER = LoggerFactory.getLogger(Queue.class);

    private String queueId;

    private String queueName;

    private String owner;

    private int cuCount;

    private String description;

    private long createTime;

    private ChargingMode chargingMode;

    private DLIClient client;

    public Queue(String queueName, String owner, int cuCount, String description,
        long createTime, ChargingMode chargingMode, DLIClient client) {
        this.queueName = queueName;
        this.owner = owner;
        this.cuCount = cuCount;
        this.description = description;
        this.createTime = createTime;
        this.chargingMode = chargingMode;
        this.client = client;
    }

    public Queue(String queueId, String queueName, String owner, int cuCount, String description,
        long createTime, ChargingMode chargingMode, DLIClient client) {
        this(queueName, owner, cuCount, description, createTime, chargingMode, client);
        this.queueId = queueId;
    }

    public Queue(String queueName, DLIClient client) {
        this.queueName = queueName;
        this.client = client;
    }

    /**
     * Delete this queue.
     *
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    public void deleteQueue() throws DLIException {
        try {
            ResponseInfo response = client.getRestApi().deleteQueue(client.getProjectId(),
                queueName, client.getAuthToken());
            if (!response.getIsSuccess()) {
                LOGGER.error(response.getMessage());
                throw new DLIException(response.getMessage());
            }
            LOGGER.info("Queue is deleted succeed, queueName: " + queueName);
        } catch (ApiException e) {
            LOGGER.error("msg", e);
            throw new DLIException(e.getMessage(), e.getCode(), e.getResponseBody());
        }
    }

    @Override
    public String toString() {
        return "Queue{"
            + "queueId='" + queueId + '\''
            + ", queueName='" + queueName + '\''
            + ", owner='" + owner + '\''
            + ", cuCount=" + cuCount
            + ", description='" + description + '\''
            + ", createTime=" + createTime
            + ", chargingMode=" + chargingMode
            + '}';
    }

}
