package com.huawei.dli.sdk.authentication;

import com.huawei.dli.restapi.JSON;
import com.huawei.dli.restapi.Pair;
import com.huawei.dli.restapi.auth.Authentication;

import com.cloud.apigateway.sdk.utils.Client;

import java.util.List;
import java.util.Map;

import okhttp3.Request;

public class AkskAuthentication implements Authentication {
    private final String accessKey;

    private final String secretKey;

    private String securityToken;

    private final String projectId;

    private final JSON json = new JSON();

    public AkskAuthentication(String accessKey, String secretKey, String projectId) {
        this.accessKey = accessKey;
        this.secretKey = secretKey;
        this.projectId = projectId;
    }

    public AkskAuthentication(String accessKey, String secretKey, String securityToken, String projectId) {
        this(accessKey, secretKey, projectId);
        this.securityToken = securityToken;
    }

    @Override
    public void applyToParams(
        String url,
        String method,
        List<Pair> queryParams,
        Map<String, String> headerParams,
        Object body) {
        com.cloud.apigateway.sdk.utils.Request apigReq = new com.cloud.apigateway.sdk.utils.Request();
        try {
            apigReq.setKey(accessKey);
            apigReq.setSecret(secretKey);
            apigReq.setUrl(url);
            apigReq.setMethod(method);
            for (Pair pair : queryParams) {
                apigReq.addQueryStringParam(pair.getName(), pair.getValue());
            }
            // TIPS: Don't set original headers to apig's request, or else will verify failed for no-get request.
            if (body != null) {
                apigReq.setBody(json.serialize(body));
            }
        } catch (Exception e) {
            throw new IllegalArgumentException(e.getMessage());
        }
        Request signReq;
        try {
            signReq = Client.signOkhttp(apigReq);
        } catch (Exception e) {
            throw new RuntimeException("Sign request failed", e);
        }
        for (Map.Entry<String, List<String>> header : signReq.headers().toMultimap().entrySet()) {
            if (!headerParams.containsKey(header.getKey())) {
                headerParams.put(header.getKey(), String.join(" ", header.getValue()));
            }
        }
        headerParams.put("X-Project-Id", projectId);
        if (securityToken != null && !securityToken.isEmpty()) {
            headerParams.put("X-Security-Token", securityToken);
        }
    }
}
