package com.huawei.dli.sdk.common;

import com.huawei.dli.sdk.common.type.TypeInfoUtils;

public class Column {

    private String columnName;

    private DataType type;

    private String description;

    private DataTypeInfo dataTypeInfo;

    private boolean isPartitionColumn;

    /**
     * Create Column instance,
     * Constructor parameters:column name, dataTypeInfo, description, isPartitionColumn.
     *
     * @param columnName        column name, e.g. column1
     * @param dataTypeInfo      dataType info with parameters,
     *                          e.g. DataTypeInfo dataTypeInfo = new DecimalTypeInfo(10, 5);
     * @param description       description for column.
     * @param isPartitionColumn column whether partition.
     */
    public Column(String columnName, DataTypeInfo dataTypeInfo, String description, boolean isPartitionColumn) {
        this.columnName = columnName;
        this.description = description;
        this.dataTypeInfo = dataTypeInfo;
        this.type = dataTypeInfo.getDataType();
        this.isPartitionColumn = isPartitionColumn;
    }

    /**
     * Create Column instance,
     * Constructor parameters:column name, dataTypeInfo, description.
     * Default isPartitionColumn is false.
     *
     * @param columnName   column name, e.g. column1
     * @param dataTypeInfo dataType info with parameters,
     *                     e.g. DataTypeInfo dataTypeInfo = new DecimalTypeInfo(10, 5);
     * @param description  description for column.
     */
    public Column(String columnName, DataTypeInfo dataTypeInfo, String description) {
        this.columnName = columnName;
        this.description = description;
        this.dataTypeInfo = dataTypeInfo;
        this.type = dataTypeInfo.getDataType();
    }

    /**
     * Create Column instance,don't need description.
     *
     * @param columnName   column name, e.g. column1
     * @param dataTypeInfo dataType info with parameters,
     *                     e.g. DataTypeInfo dataTypeInfo = new DecimalTypeInfo(10, 5);
     */
    public Column(String columnName, DataTypeInfo dataTypeInfo) {
        this.columnName = columnName;
        this.dataTypeInfo = dataTypeInfo;
        this.type = dataTypeInfo.getDataType();
    }

    /**
     * Create Column instance,
     * Constructor parameters:column name, data type, description.
     * Default isPartitionColumn is false.
     *
     * @param columnName  column name, e.g. column1
     * @param type        data type, e.g. DataType.INT
     * @param description description for column.
     */
    public Column(String columnName, DataType type, String description) {
        this.columnName = columnName;
        this.type = type;
        this.description = description;
        this.dataTypeInfo = TypeInfoUtils.getPrimitiveTypeInfo(type);
    }

    /**
     * Create Column instance,
     * Constructor parameters:column name, data type, description, isPartitionColumn.
     *
     * @param columnName        column name, e.g. column1
     * @param type              data type, e.g. DataType.INT
     * @param description       description for column.
     * @param isPartitionColumn column whether partition.
     */
    public Column(String columnName, DataType type, String description, boolean isPartitionColumn) {
        this.columnName = columnName;
        this.type = type;
        this.description = description;
        this.dataTypeInfo = TypeInfoUtils.getPrimitiveTypeInfo(type);
        this.isPartitionColumn = isPartitionColumn;
    }

    /**
     * Create Column instance,don't need description.
     *
     * @param columnName column name, e.g. column1
     * @param type       data type, e.g. DataType.INT
     */
    public Column(String columnName, DataType type) {
        this.columnName = columnName;
        this.type = type;
        this.dataTypeInfo = TypeInfoUtils.getPrimitiveTypeInfo(type);
    }

    public String getColumnName() {
        return columnName;
    }

    public DataType getType() {
        return type;
    }

    public String getDescription() {
        return description;
    }

    public DataTypeInfo getDataTypeInfo() {
        return dataTypeInfo;
    }

    public boolean isPartitionColumn() {
        return isPartitionColumn;
    }

    @Override
    public String toString() {
        return "Column{"
            + "columnName='" + columnName + '\''
            + ", type=" + type
            + ", description='" + description + '\''
            + ", isPartitionColumn='" + isPartitionColumn + '\''
            + '}';
    }

}
