package com.huawei.dli.sdk.common;

import lombok.Getter;
import lombok.Setter;

@Getter
@Setter
public class DLIInfo {
    public static final String IAM_ENDPOINT_PREFIX = "iam.";

    public static final String OBS_ENDPOINT = "obs.myhuaweicloud.com";

    public static final String SERVER_ENDPOINT_PREFIX = "dli.";

    public static final String ENDPOINT_POSTFIX = ".myhuaweicloud.com";

    public static final String SERVER_NAME = "DLI";

    private String iamEndPoint;

    private int iamPort;

    private String obsEndPoint;

    private int obsPort;

    private String serverEndPoint;

    private int serverPort;

    private String regionName;

    private String serverName = SERVER_NAME;

    private String domainName;

    private String userName;

    private String password;

    private String projectId;

    private String accessKey;

    private String secretKey;

    private String securityToken;

    private String token;

    private String urlPrefix;

    private String proxyHost;

    private int proxyPort = 0;

    private String proxyUser;

    private String proxyPwd;

    private boolean useRetry = false;

    private int retryTimes = 3;

    private int retryIntervals = 30;

    private int partThreadNum = 5;

    public DLIInfo(String regionName, String projectId, String token) {
        this.iamEndPoint = IAM_ENDPOINT_PREFIX + regionName + ENDPOINT_POSTFIX;
        this.obsEndPoint = OBS_ENDPOINT;
        this.serverEndPoint = SERVER_ENDPOINT_PREFIX + regionName + ENDPOINT_POSTFIX;
        this.regionName = regionName;
        this.token = token;
        this.projectId = projectId;
    }

    public DLIInfo(String regionName, String domainName, String userName, String password, String projectId) {
        this.iamEndPoint = IAM_ENDPOINT_PREFIX + regionName + ENDPOINT_POSTFIX;
        this.obsEndPoint = OBS_ENDPOINT;
        this.serverEndPoint = SERVER_ENDPOINT_PREFIX + regionName + ENDPOINT_POSTFIX;
        this.regionName = regionName;
        this.domainName = domainName;
        this.userName = userName;
        this.password = password;
        this.projectId = projectId;
    }

    public DLIInfo(String regionName, String accessKey, String secretKey, String projectId) {
        this.obsEndPoint = OBS_ENDPOINT;
        this.serverEndPoint = SERVER_ENDPOINT_PREFIX + regionName + ENDPOINT_POSTFIX;
        this.regionName = regionName;
        this.accessKey = accessKey;
        this.secretKey = secretKey;
        this.projectId = projectId;
    }

    public DLIInfo() {
    }

    public void setProxy(String proxyHost, int proxyPort) {
        this.proxyHost = proxyHost;
        this.proxyPort = proxyPort;
    }

    public void setProxy(String proxyHost, int proxyPort, String proxyUser, String proxyPwd) {
        this.proxyHost = proxyHost;
        this.proxyPort = proxyPort;
        this.proxyUser = proxyUser;
        this.proxyPwd = proxyPwd;
    }
}
