package com.huawei.dli.sdk.common;

import com.huawei.dli.sdk.common.type.TypeInfoUtils;

public enum DataType {

    INT("int", 0),
    SMALLINT("smallint", 1),
    BIGINT("bigint", 2),
    LONG("long", 3),
    SHORT("short", 4),
    FLOAT("float", 5),
    DOUBLE("double", 6),
    DECIMAL("decimal", 7),
    BOOLEAN("boolean", 8),
    STRING("string", 9),
    DATE("date", 10),
    TIMESTAMP("timestamp", 11),
    TINYINT("tinyint", 12),
    STRUCT("struct", 13),
    ARRAY("array", 14),
    MAP("map", 15),
    UNKNOWN("unknown", 16);

    private final int value;

    private final String name;

    DataType(String name, int value) {
        this.name = name;
        this.value = value;
    }

    public String getName() {
        return name;
    }

    public int getValue() {
        return value;
    }

    public static DataType fromName(String name) {
        DataTypeInfo typeInfo = TypeInfoUtils.getDataTypeInfo(name);
        return typeInfo.getDataType();
    }

    private static int[] getParams(String paramStr) {
        int index1 = paramStr.indexOf("(");
        int index2 = paramStr.indexOf(",");
        int index3 = paramStr.indexOf(")");
        String paramStr1 = paramStr.substring(index1 + 1, index2);
        String paramStr2 = paramStr.substring(index2 + 1, index3);
        int param1 = Integer.parseInt(paramStr1.trim());
        int param2 = Integer.parseInt(paramStr2.trim());
        return new int[] {param1, param2};
    }

    public static DataTypeInfo getDataTypeInfo(DataType dataType, String paramStr) {
        if (DataType.DECIMAL.equals(dataType)) {
            if (paramStr == null) {
                return new DecimalTypeInfo();
            }
            int[] params = dataType.getParams(paramStr);
            return params[0] == DecimalTypeInfo.DEFAULT_PRECISION && params[1] == DecimalTypeInfo.DEFAULT_SCALE
                ? new DecimalTypeInfo() : new DecimalTypeInfo(params[0], params[1]);
        } else {
            return new SimplePrimitiveTypeInfo(dataType);
        }
    }
}
