package com.huawei.dli.sdk.common;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class DecimalTypeInfo extends AbstractPrimitiveTypeInfo {

    private static final Logger LOGGER = LoggerFactory.getLogger(DecimalTypeInfo.class);

    public static final int DEFAULT_PRECISION = 38;

    public static final int DEFAULT_SCALE = 5;

    private int precision;

    private int scale;

    public DecimalTypeInfo() {
        this(DEFAULT_PRECISION, DEFAULT_SCALE);
    }

    public DecimalTypeInfo(int precision, int scale) {
        super(DataType.DECIMAL);
        this.validateParameter(precision, scale);
        this.precision = precision;
        this.scale = scale;
    }

    public String toString() {
        return String.format("%s(%s,%s)", super.getDataType().getName(), this.precision, this.scale);
    }

    private void validateParameter(int precision, int scale) {
        if (precision >= 1 && precision <= 38) {
            if (scale >= 0 && scale <= 37) {
                if (scale > precision) {
                    LOGGER.error("Decimal precision must be larger than or equal to scale");
                    throw new IllegalArgumentException("Decimal precision must be larger than or equal to scale");
                }
            } else {
                LOGGER.error("Decimal scale " + scale + " out of range: [0, " + 37 + "]");
                throw new IllegalArgumentException("Decimal scale " + scale + " out of range: [0, " + 37 + "]");
            }
        } else {
            LOGGER.error("Decimal precision " + precision + " out of range: [1, " + 38 + "]");
            throw new IllegalArgumentException("Decimal precision " + precision + " out of range: [1, " + 38 + "]");
        }
    }

    public int getPrecision() {
        return this.precision;
    }

    public int getScale() {
        return this.scale;
    }

}
