package com.huawei.dli.sdk.common;

import com.huawei.dli.restapi.model.JobListInfo;
import com.huawei.dli.restapi.model.JobTagEntityInfo;
import com.huawei.dli.restapi.model.StatusResult;

import java.util.HashMap;
import java.util.Map;

public class JobResultInfo {

    private JobStatus jobStatus;

    private String detail;

    private String statement;

    private Long duration;

    private Long startTime;

    private Long endTime;

    private Long inputSize;

    private Long resultCount;

    private String databaseName;

    private String tableName;

    private String queueName;

    private Boolean withColumnHeader;

    private String owner;

    private JobType jobType;

    private String message;

    private String jobId;

    private Boolean isSuccess;

    private Map<String, String> tags;

    public JobResultInfo(StatusResult statusResult) {
        this.jobStatus = JobStatus.fromName(statusResult.getStatus());
        this.detail = statusResult.getDetail();
        this.statement = statusResult.getStatement();
        this.duration = statusResult.getDuration();
        this.startTime = statusResult.getStartTime();
        this.inputSize = statusResult.getInputSize();
        this.resultCount = statusResult.getResultCount();
        this.databaseName = statusResult.getDatabaseName();
        this.tableName = statusResult.getTableName();
        this.queueName = statusResult.getQueueName();
        this.owner = statusResult.getOwner();
        try {
            this.jobType = JobType.valueOf(statusResult.getJobType());
        } catch (IllegalArgumentException e) {
            this.jobType = JobType.UNKNOWN;
        }
        this.message = statusResult.getMessage();
        this.jobId = statusResult.getJobId();
        this.isSuccess = statusResult.getIsSuccess();
        if (statusResult.getTags() != null) {
            this.tags = new HashMap<>();
            for (JobTagEntityInfo tagEntityInfo : statusResult.getTags()) {
                this.tags.put(tagEntityInfo.getKey(), tagEntityInfo.getValue());
            }
        }
    }

    public JobResultInfo(JobListInfo jobListInfo) {
        this.jobStatus = JobStatus.fromName(jobListInfo.getStatus());
        this.detail = jobListInfo.getDetail();
        this.statement = jobListInfo.getStatement();
        this.duration = jobListInfo.getDuration();
        this.startTime = jobListInfo.getStartTime();
        this.endTime = jobListInfo.getEndTime();
        this.inputSize = jobListInfo.getInputSize();
        this.resultCount = jobListInfo.getResultCount();
        this.databaseName = jobListInfo.getDatabaseName();
        this.tableName = jobListInfo.getTableName();
        this.queueName = jobListInfo.getQueueName();
        this.withColumnHeader = jobListInfo.getWithColumnHeader();
        this.owner = jobListInfo.getOwner();
        try {
            this.jobType = JobType.valueOf(jobListInfo.getJobType());
        } catch (IllegalArgumentException e) {
            this.jobType = JobType.UNKNOWN;
        }
        this.message = jobListInfo.getMessage();
        this.jobId = jobListInfo.getJobId();
        if (jobListInfo.getTags() != null) {
            this.tags = new HashMap<>();
            for (JobTagEntityInfo tagEntityInfo : jobListInfo.getTags()) {
                this.tags.put(tagEntityInfo.getKey(), tagEntityInfo.getValue());
            }
        }
    }

    public JobStatus getJobStatus() {
        return jobStatus;
    }

    public String getDetail() {
        return detail;
    }

    public String getStatement() {
        return statement;
    }

    public Long getDuration() {
        return duration;
    }

    public Long getStartTime() {
        return startTime;
    }

    public Long getEndTime() {
        return endTime;
    }

    public Long getInputSize() {
        return inputSize;
    }

    public Long getResultCount() {
        return resultCount;
    }

    public String getDatabaseName() {
        return databaseName;
    }

    public String getTableName() {
        return tableName;
    }

    public String getQueueName() {
        return queueName;
    }

    public Boolean getWithColumnHeader() {
        return withColumnHeader;
    }

    public String getOwner() {
        return owner;
    }

    public JobType getJobType() {
        return jobType;
    }

    public String getMessage() {
        return message;
    }

    public String getJobId() {
        return jobId;
    }

    public Boolean getSuccess() {
        return isSuccess;
    }

    public Map<String, String> getTags() {
        return tags;
    }

    @Override
    public String toString() {
        return "JobResultInfo{"
            + "jobStatus=" + jobStatus
            + ", detail='" + detail + '\''
            + ", statement='" + statement + '\''
            + ", duration=" + duration
            + ", startTime=" + startTime
            + ", endTime=" + endTime
            + ", inputSize=" + inputSize
            + ", resultCount=" + resultCount
            + ", databaseName='" + databaseName + '\''
            + ", tableName='" + tableName + '\''
            + ", queueName='" + queueName + '\''
            + ", withColumnHeader=" + withColumnHeader
            + ", owner='" + owner + '\''
            + ", jobType=" + jobType
            + ", message='" + message + '\''
            + ", jobId='" + jobId + '\''
            + ", isSuccess=" + isSuccess
            + '}';
    }
}
