package com.huawei.dli.sdk.common;

import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class PartitionSpec {
    private static final String COMMA = ",";

    private static final String SLASH = "/";

    private final Map<String, String> kv = new LinkedHashMap<String, String>();

    public PartitionSpec() {
    }

    public PartitionSpec(String spec) {
        if (spec == null) {
            throw new IllegalArgumentException("Argument \'spec\' cannot be null");
        }
        String[] groups = spec.split("[,/]");
        for (String group : groups) {
            String[] keyValue = group.split("=");
            if (keyValue.length != 2) {
                throw new IllegalArgumentException("Invalid partition spec.");
            }

            String k = keyValue[0].trim();
            String v = keyValue[1].trim().replaceAll("'", "").replaceAll("\"", "");
            if (k.length() == 0 || v.length() == 0) {
                throw new IllegalArgumentException("Invalid partition spec.");
            }

            set(k, v);
        }
    }

    public void set(String key, String value) {
        kv.put(key, value);
    }

    public String get(String key) {
        return kv.get(key);
    }

    public Set<String> keys() {
        return kv.keySet();
    }

    public boolean isEmpty() {
        return kv.isEmpty();
    }

    @Override
    public String toString() {
        return toString(true, false);
    }

    public String toString(boolean quote, boolean useSlashDelimiter) {
        String delimiter = useSlashDelimiter ? SLASH : COMMA;

        List<String> entries = new LinkedList<>();
        String[] keys = keys().toArray(new String[0]);

        for (String key : keys) {
            StringBuilder entryBuilder = new StringBuilder();
            entryBuilder.append(key).append("=");
            if (quote) {
                entryBuilder.append("\'").append(kv.get(key)).append("\'");
            } else {
                entryBuilder.append(kv.get(key));
            }
            entries.add(entryBuilder.toString());
        }

        return String.join(delimiter, entries);
    }

    public Map<String, String> getKv() {
        return kv;
    }
}
