package com.huawei.dli.sdk.common;

import com.huawei.dli.sdk.exception.DLIException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class TableSchema {

    private static final Logger LOGGER = LoggerFactory.getLogger(TableSchema.class);

    private List<Column> columns;

    private List<String> sortColumns;

    private String createTableSql;

    private String tableComment;

    public String getTableComment() {
        return tableComment;
    }

    public void setTableComment(String tableComment) {
        this.tableComment = tableComment;
    }

    public String getCreateTableSql() {
        return createTableSql;
    }

    public void setCreateTableSql(String createTableSql) {
        this.createTableSql = createTableSql;
    }

    public List<String> getSortColumns() {
        return sortColumns;
    }

    public void setSortColumns(List<String> sortColumns) {
        this.sortColumns = sortColumns;
    }

    public List<Column> getColumns() {
        return columns;
    }

    public void setColumns(List<Column> columns) {
        this.columns = columns;
    }

    public void putColumn(Column column) {
        if (columns == null) {
            columns = new ArrayList<Column>();
        }
        columns.add(column);
    }

    public List<Column> getColumn() {
        return columns;
    }

    public Column getColumn(int index) {
        return columns.get(index);
    }

    public Column getColumn(String name) throws DLIException {
        if (columns == null || columns.size() == 0) {
            LOGGER.error("columns is null or no element");
            throw new DLIException("columns is null or no element");
        }
        for (Column column : columns) {
            if (name.equalsIgnoreCase(column.getColumnName())) {
                return column;
            }
        }
        LOGGER.error(name + " column is not exist");
        throw new DLIException(name + " column is not exist");
    }

    public boolean containsColumn(String name) throws DLIException {
        if (columns == null || columns.size() == 0) {
            LOGGER.error("columns is null or no element");
            throw new DLIException("columns is null or no element");
        }
        for (Column column : columns) {
            if (name.equalsIgnoreCase(column.getColumnName())) {
                return true;
            }
        }
        return false;
    }

    public int getColumnIdx(String name) throws DLIException {
        if (columns == null || columns.size() == 0) {
            LOGGER.error("columns is null or no element");
            throw new DLIException("columns is null or no element");
        }
        int index = 0;
        for (Column column : columns) {
            if (name.equalsIgnoreCase(column.getColumnName())) {
                return index;
            }
            index++;
        }
        return -1;
    }

}
