package com.huawei.dli.sdk.common.type;

import com.huawei.dli.sdk.common.DataTypeInfo;
import com.huawei.dli.sdk.exception.DLIException;

import java.util.List;

public interface DliStruct {
    /**
     * Get the field number of this struct.
     *
     * @return the field number of this struct
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    int getFieldCount() throws DLIException;

    /**
     * Get the field name by the index of the struct
     *
     * @param index struct index
     * @return String
     */
    String getFieldName(int index);

    /**
     * Get the field type info by the index of the struct
     *
     * @param index struct index
     * @return DataTypeInfo
     */
    DataTypeInfo getFieldTypeInfo(int index);

    /**
     * Get the field value by the index of the struct
     *
     * @param index struct index
     * @return Object
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    Object getFieldValue(int index) throws DLIException;

    /**
     * Get the field type info the field name of the struct
     *
     * @param fieldName the field name of the struct
     * @return DataTypeInfo
     */
    DataTypeInfo getFieldTypeInfo(String fieldName);

    /**
     * Get the field value by the field name of the struct
     *
     * @param fieldName the field name of the struct
     * @return Object
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    Object getFieldValue(String fieldName) throws DLIException;

    /**
     * Get the List of field values of the struct
     *
     * @return List
     * @throws DLIException If fail to call the API, e.g. server error or cannot deserialize the response body
     */
    List<Object> getFieldValues() throws DLIException;
}
