package com.huawei.dli.sdk.common.type;

import com.huawei.dli.sdk.common.DataTypeInfo;
import com.huawei.dli.sdk.exception.DLIException;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonParser;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class SimpleArray implements DliArray {

    private static final Logger LOGGER = LoggerFactory.getLogger(SimpleArray.class);

    protected ArrayTypeInfo typeInfo;

    protected List<Object> values;

    protected String data;

    SimpleArray(ArrayTypeInfo typeInfo, String data) {
        if (typeInfo == null) {
            LOGGER.error("Illegal arguments for StructObject.");
            throw new IllegalArgumentException("Illegal arguments for StructObject.");
        }
        this.typeInfo = typeInfo;
        this.data = data;
    }

    public DataTypeInfo getElementTypeInfo() {
        return this.typeInfo.getElementTypeInfo();
    }

    public List<Object> getElementValues() throws DLIException {
        if (values == null) {
            parseArray();
        }
        return values;
    }

    private void parseArray() throws DLIException {
        JsonArray jsonArray = JsonParser.parseString(data).getAsJsonArray();
        DataTypeInfo elementTypeInfo = typeInfo.getElementTypeInfo();
        this.values = new ArrayList<>();

        for (JsonElement elem : jsonArray) {
            String fieldObj = elem.isJsonPrimitive() ? elem.getAsString() : elem.toString();
            this.values.add(TypeUtils.transform(fieldObj, elementTypeInfo));
        }
    }

    public String toString() {
        return this.data;
    }
}
