package com.huawei.dli.sdk.common.type;

import com.huawei.dli.sdk.common.DataTypeInfo;
import com.huawei.dli.sdk.exception.DLIException;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.HashMap;
import java.util.Map;

public class SimpleMap implements DliMap {
    private static final Logger LOGGER = LoggerFactory.getLogger(SimpleMap.class);

    protected MapTypeInfo typeInfo;

    protected Map values;

    protected String data;

    SimpleMap(MapTypeInfo typeInfo, String data) {
        if (typeInfo == null) {
            LOGGER.error("Illegal arguments for StructObject.");
            throw new IllegalArgumentException("Illegal arguments for StructObject.");
        }
        this.typeInfo = typeInfo;
        this.data = data;
    }

    public DataTypeInfo getKeyTypeInfo() {
        return this.typeInfo.getKeyTypeInfo();
    }

    public DataTypeInfo getValueTypeInfo() {
        return this.typeInfo.getValueTypeInfo();
    }

    public Map getValues() throws DLIException {
        if (values == null) {
            parseMap();
        }
        return values;
    }

    @SuppressWarnings("unchecked")
    private void parseMap() throws DLIException {
        JsonObject jsonObject = JsonParser.parseString(data).getAsJsonObject();
        values = new HashMap<>();
        for (Map.Entry<String, JsonElement> kv : jsonObject.entrySet()) {
            Object entryKey = TypeUtils.transform(kv.getKey(), getKeyTypeInfo());
            JsonElement elem = kv.getValue();
            String fieldObj = elem.isJsonPrimitive() ? elem.getAsString() : elem.toString();
            Object entryValue = TypeUtils.transform(fieldObj, getValueTypeInfo());
            values.put(entryKey, entryValue);
        }
    }

    public String toString() {
        return this.data;
    }
}
