package com.huawei.dli.sdk.common.type;

import com.huawei.dli.sdk.common.DataTypeInfo;
import com.huawei.dli.sdk.exception.DLIException;

import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

public class SimpleStruct implements DliStruct {
    private static final Logger LOGGER = LoggerFactory.getLogger(SimpleStruct.class);

    protected StructTypeInfo typeInfo;

    protected List<Object> values;

    protected String data;

    SimpleStruct(StructTypeInfo typeInfo, String data) {
        if (typeInfo == null) {
            LOGGER.error("Illegal arguments for StructObject.");
            throw new IllegalArgumentException("Illegal arguments for StructObject.");
        }
        this.typeInfo = typeInfo;
        this.data = data;
    }

    public int getFieldCount() throws DLIException {
        if (values == null) {
            parseStruct();
        }
        return values.size();
    }

    public String getFieldName(int index) {
        return typeInfo.getFieldNames().get(index);
    }

    public DataTypeInfo getFieldTypeInfo(int index) {
        return typeInfo.getFieldTypeInfos().get(index);
    }

    public Object getFieldValue(int index) throws DLIException {
        return getFieldValues().get(index);
    }

    public DataTypeInfo getFieldTypeInfo(String fieldName) {
        for (int i = 0; i < typeInfo.getFieldCount(); ++i) {
            if (typeInfo.getFieldNames().get(i).equalsIgnoreCase(fieldName)) {
                return typeInfo.getFieldTypeInfos().get(i);
            }
        }
        return null;
    }

    public Object getFieldValue(String fieldName) throws DLIException {
        for (int i = 0; i < typeInfo.getFieldCount(); ++i) {
            if (typeInfo.getFieldNames().get(i).equalsIgnoreCase(fieldName)) {
                return getFieldValues().get(i);
            }
        }
        return null;
    }

    public List<Object> getFieldValues() throws DLIException {
        if (values == null) {
            parseStruct();
        }
        return values;
    }

    private void parseStruct() throws DLIException {
        JsonObject jsonObject = JsonParser.parseString(data).getAsJsonObject();
        values = new ArrayList<>();

        int fieldIdx = 0;
        for (String fieldName : typeInfo.getFieldNames()) {
            JsonElement elem = jsonObject.get(fieldName);
            String fieldObj = elem.isJsonPrimitive() ? elem.getAsString() : elem.toString();
            DataTypeInfo fieldTypeInfo = typeInfo.getFieldTypeInfos().get(fieldIdx);

            values.add(TypeUtils.transform(fieldObj, fieldTypeInfo));
            fieldIdx++;
        }
    }

    public String toString() {
        return this.data;
    }
}
