package com.huawei.dli.sdk.common.type;

import com.huawei.dli.sdk.common.DataType;
import com.huawei.dli.sdk.common.DataTypeInfo;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

public class SimpleStructTypeInfo implements StructTypeInfo {
    private static final Logger LOGGER = LoggerFactory.getLogger(SimpleStructTypeInfo.class);

    private List<String> fieldNames;

    private List<DataTypeInfo> fieldTypeInfos;

    SimpleStructTypeInfo(List<String> names, List<DataTypeInfo> typeInfos) {
        this.validateParameters(names, typeInfos);
        this.fieldNames = this.toLowerCase(names);
        this.fieldTypeInfos = new ArrayList<>(typeInfos);
    }

    private List<String> toLowerCase(List<String> names) {
        List<String> lowerNames = new ArrayList<String>(names.size());
        for (String name : names) {
            lowerNames.add(name.toLowerCase(Locale.US));
        }
        return lowerNames;
    }

    private void validateParameters(List<String> names, List<DataTypeInfo> typeInfos) {
        if (names != null && typeInfos != null && !names.isEmpty() && !typeInfos.isEmpty()) {
            if (names.size() != typeInfos.size()) {
                LOGGER.error("The amount of field names and types must be same");
                throw new IllegalArgumentException("The amount of field names and types must be same");
            }
        } else {
            LOGGER.error("Invalid name or element type for struct.");
            throw new IllegalArgumentException("Invalid name or element type for struct.");
        }
    }

    public String getTypeName() {
        StringBuilder stringBuilder = new StringBuilder(getDataType().getName());
        stringBuilder.append("<");

        for (int i = 0; i < fieldNames.size(); ++i) {
            if (i > 0) {
                stringBuilder.append(",");
            }
            stringBuilder.append(fieldNames.get(i));
            stringBuilder.append(":");
            stringBuilder.append(fieldTypeInfos.get(i).toString());
        }
        stringBuilder.append(">");
        return stringBuilder.toString();
    }

    public List<String> getFieldNames() {
        return fieldNames;
    }

    public List<DataTypeInfo> getFieldTypeInfos() {
        return fieldTypeInfos;
    }

    public int getFieldCount() {
        return fieldNames.size();
    }

    public DataType getDataType() {
        return DataType.STRUCT;
    }

    public String toString() {
        return this.getTypeName();
    }
}
