package com.huawei.dli.sdk.common.utils;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.function.Function;

public class IterableInputStream<T> extends InputStream {
    public static final int EOF = -1;

    private static final InputStream EOF_INPUT_STREAM = new InputStream() {
        @Override
        public int read() throws IOException {
            return EOF;
        }
    };

    private final Iterator<T> iterator;

    private final Function<T, byte[]> dataMapper;

    private final Function<T, Integer> lenMappper;

    private InputStream current;

    public IterableInputStream(Iterator<T> iterator, Function<T, byte[]> dataMapper, Function<T, Integer> lenMapper) {
        this.iterator = iterator;
        this.dataMapper = dataMapper;
        this.lenMappper = lenMapper;
        next();
    }

    @Override
    public int read() throws IOException {
        int n = current.read();
        while (n == EOF && current != EOF_INPUT_STREAM) {
            next();
            n = current.read();
        }
        return n;
    }

    private void next() {
        if (iterator.hasNext()) {
            T data = iterator.next();
            current = new ByteArrayInputStream(dataMapper.apply(data), 0, lenMappper.apply(data));
        } else {
            current = EOF_INPUT_STREAM;
        }
    }
}
