package com.huawei.dli.sdk.common.utils;

import java.sql.SQLException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Locale;
import java.util.Set;
import java.util.regex.Pattern;

public class SqlUtils {
    private static String[] noResultQueryKeywords = new String[] {
        "INSERT", "LOAD", "UPDATE", "DELETE", "MSCK",
        "DROP", "CREATE", "ALTER", "TRUNCATE", "RENAME",
        "SET", "RESET", "ANALYZE", "REFRESH", "CACHE", "UNCACHE", "CLEAR", "GRANT", "REVOKE"
    };

    private static Pattern keywordPattern = Pattern.compile("(.*INSERT\\s+(INTO|OVERWRITE).*)|(.*LOAD\\s+DATA.*)");

    private static Set<String> noResultKeywordSet = new HashSet<>(Arrays.asList(noResultQueryKeywords));

    public static boolean isQuery(String sql) {
        sql = sql.replaceAll("\n", " ").replaceAll("\r", " ").toUpperCase(Locale.US);
        if (keywordPattern.matcher(sql).matches()) {
            return false;
        }
        String[] items = sql.split("\\s");
        if (items.length > 0) {
            String startWord = items[0];
            return !noResultKeywordSet.contains(startWord);
        }
        return false;
    }

    public static void checkForNoResult(String sql) throws SQLException {
        String[] items = sql.split("\\s");
        if (items.length > 0) {
            String startWord = items[0].toUpperCase(Locale.US);
            if (noResultKeywordSet.contains(startWord)) {
                throw new SQLException("Can not issue DML/DDL/DCL statements with executeQuery()");
            }
        }
    }
}
