package com.huawei.dli.sdk.exception;

import com.huawei.dli.restapi.ApiException;

import com.google.gson.JsonObject;
import com.google.gson.JsonParser;

public class DLIException extends Exception {

    private int code = 0;

    private String responseBody = null;

    public DLIException() {
        super();
    }

    public DLIException(String message) {
        super(message);
    }

    public DLIException(String message, Throwable cause) {
        super(message, cause);
    }

    public DLIException(Throwable cause) {
        super(cause);
    }

    public DLIException(ApiException e) {
        this(e.getMessage(), e.getCode(), e.getResponseBody());
    }

    public DLIException(String message, int code, String responseBody) {
        this(errorMessageParser(code, message, responseBody));
        this.code = code;
        this.responseBody = responseBody;
    }

    public DLIException(int code, String message) {
        this(message);
        this.code = code;
    }

    public int getCode() {
        return code;
    }

    public String getResponseBody() {
        return responseBody;
    }

    private static String errorMessageParser(int code, String message, String responseBody) {
        if (code == 0) {
            return message;
        }
        if (code == 400) {
            if (responseBody.contains("error_code") && responseBody.contains("error_msg")) {
                if (responseBody.startsWith("[") && responseBody.endsWith("]")) {
                    return "HTTP Status:" + code + " " + message + "; " + responseBody;
                }
                return "HTTP Status:" + code + " " + message + "; " + getJsonString(responseBody, "error_code")
                    + " " + getJsonString(responseBody, "error_msg");
            }
            if (responseBody.contains("message")) {
                return "HTTP Status:" + code + " " + message + "; " + getJsonString(responseBody, "error.message");
            } else {
                return "HTTP Status:" + code + " " + message + "; " + responseBody;
            }
        }
        if (code == 401) {
            if (responseBody.contains("error_code") && responseBody.contains("error_msg")) {
                return "HTTP Status:" + code + " " + message + "; " + getJsonString(responseBody, "error_code")
                    + " " + getJsonString(responseBody, "error_msg");
            }
            if (responseBody.contains("message")) {
                return "HTTP Status:" + code + " " + message + "; " + getJsonString(responseBody, "error.message");
            } else {
                return code + " " + message + ";" + responseBody;
            }
        }
        if (code == 403) {
            if (responseBody.contains("<title>")) {
                int i1 = responseBody.indexOf("<title>");
                int i2 = responseBody.indexOf("</title>");
                return "HTTP Status:" + code + " " + message + "; " + responseBody.substring(i1 + 7, i2);
            }
            return "HTTP Status:" + code + " " + message + "; " + responseBody;
        }
        if (code == 404) {
            if (responseBody.contains("<title>")) {
                int i1 = responseBody.indexOf("<title>");
                int i2 = responseBody.indexOf("</title>");
                return "HTTP Status:" + code + " " + message + "; " + responseBody.substring(i1 + 7, i2);
            }
            return responseBody;
        }
        return "HTTP Status:" + code + " " + message + "; " + responseBody;
    }

    private static String getJsonString(String json, String key) {
        String content = "";
        JsonObject root = JsonParser.parseString(json).getAsJsonObject();
        for (String tk : key.split("[.]")) {
            if (root != null && root.has(tk)) {
                if (root.get(tk) instanceof JsonObject) {
                    root = (JsonObject) root.get(tk);
                } else {
                    content = root.get(tk).getAsString();
                }
            }
        }
        return content;
    }

}
